from javax.swing import *
from javax.swing.border import LineBorder
from java.awt import GridLayout, BorderLayout, FlowLayout, Color, Dimension, Canvas
from visad import ScalarMapListener, ScalarMapEvent, Set
import jarray

gray_cmd  = 'Gray'
color_cmd = 'Color'
igray_cmd = 'Inverse Gray'


class ColorScaleSelect(JMenu, ScalarMapListener):  
  def __init__(self, menuName, rgbMap, defaultScale=None):
    JMenu.__init__(self,menuName)
    self.rgbMap = rgbMap
    self.colorControl = None
    self.initialized = 0

    if defaultScale == None:
      self.current = igray_cmd
    else:
      self.current = defaultScale

    self.setupGUI()

    if self.rgbMap != None:
      self.colorControl = self.rgbMap.getControl()
      self.rgbMap.addScalarMapListener(self)
      if self.colorControl != None:
        self.setup()

  def setup(self):
    self.initialized = 1
    n_comps  = self.colorControl.getNumberOfComponents()
    n_colors = self.colorControl.getNumberOfColors()
                                                                                                                                        
    self.clr_table = []
    for i in xrange(n_comps):
      self.clr_table.append(jarray.zeros(n_colors, 'f'))
    self.clr_table = self.colorControl.initTableVis5D(self.clr_table)
                                                                                                                                        
    self.gry_table = []
    for i in xrange(n_comps):
      self.gry_table.append(jarray.zeros(n_colors, 'f'))
    self.gry_table = self.colorControl.initTableGreyWedge(self.gry_table)
                                                                                                                                        
    self.igry_table = []
    for i in xrange(n_comps):
      self.igry_table.append(jarray.zeros(n_colors, 'f'))
    self.igry_table = self.colorControl.initTableGreyWedge(self.igry_table, 1)
                                                                                                                                        
    self.n_comps = n_comps
    self.n_colors = n_colors

    if self.current == igray_cmd:
      self.doInvGrey()
      self.invert = 1
      self.igrb.setSelected(1)
    elif self.current == gray_cmd:
      self.doGrey()
      self.invert = 0
      self.ggrb.setSelected(1)
    elif self.current == color_cmd:
      self.doColor()
      self.invert = 0
      self.ccrb.setSelected(1)


  def setupGUI(self):
    bg = ButtonGroup()
                                                                                                                                        
    ccrb = JRadioButtonMenuItem(color_cmd, actionPerformed=self.scaleSelect)
    ccrb.setEnabled(1)
    bg.add(ccrb)
    self.add(ccrb)
    self.ccrb = ccrb
                                                                                                                                        
    ggrb = JRadioButtonMenuItem(gray_cmd, actionPerformed=self.scaleSelect)
    ggrb.setEnabled(1)
    bg.add(ggrb)
    self.add(ggrb)
    self.ggrb = ggrb
                                                                                                                                        
    igrb = JRadioButtonMenuItem(igray_cmd, actionPerformed=self.scaleSelect)
    igrb.setEnabled(1)
    bg.add(igrb)
    self.add(igrb)
    self.igrb = igrb

      
  
  def setupColorTables(self):
    if self.rgbMap != None:
      self.colorControl = self.rgbMap.getControl()
      self.rgbMap.addScalarMapListener(self)
      n_comps  = self.colorControl.getNumberOfComponents()
      n_colors = self.colorControl.getNumberOfColors()
      
      self.clr_table = []
      for i in xrange(n_comps):
        self.clr_table.append(jarray.zeros(n_colors, 'f'))
      self.clr_table = self.colorControl.initTableVis5D(self.clr_table)
      
      self.gry_table = []
      for i in xrange(n_comps):
        self.gry_table.append(jarray.zeros(n_colors, 'f'))
      self.gry_table = self.colorControl.initTableGreyWedge(self.gry_table)  
      
      self.igry_table = []
      for i in xrange(n_comps):
        self.igry_table.append(jarray.zeros(n_colors, 'f'))
      self.igry_table = self.colorControl.initTableGreyWedge(self.igry_table, 1)  
      
      
      self.n_comps = n_comps
      self.n_colors = n_colors
      
  def scaleSelect(self, event):
    cmd = event.getActionCommand()
    if cmd == gray_cmd and self.current != gray_cmd:
      self.doGrey()
      self.invert = 0
      self.ggrb.setSelected(1)
    if cmd == color_cmd and self.current != color_cmd:
      self.doColor()
      self.invert = 0
      self.ccrb.setSelected(1)
    if cmd == igray_cmd and self.current != igray_cmd:
      self.doInvGrey()
      self.invert = 1
      self.igrb.setSelected(1)
    self.current = cmd
    
  def doGrey(self):
    if self.rgbMap != None:
      fnc = self.colorControl.getFunction()
      if fnc != None:
        self.doMaskColorFunction(fnc, self.gry_table)
        self.colorControl.setFunction(fnc)
      else:
        self.colorControl.initGreyWedge()
        self.colorControl.changeControl(1)
      self.currentTable = self.gry_table

      
  def doInvGrey(self):
    if self.rgbMap != None:
      fnc = self.colorControl.getFunction()
      if fnc != None:
        self.doMaskColorFunction(fnc, self.igry_table)
        self.colorControl.setFunction(fnc)
      else:
        self.colorControl.initGreyWedge(1)
        self.colorControl.changeControl(1)
      self.currentTable = self.igry_table
    
  def doColor(self):
    if self.rgbMap != None:
      fnc = self.colorControl.getFunction()
      if fnc != None:
        self.doMaskColorFunction(fnc, self.clr_table)
        self.colorControl.setFunction(fnc)
      else:
        self.colorControl.initVis5D()
        self.colorControl.changeControl(1)
      self.currentTable = self.clr_table
      
      
  def changeColorScalarMap(self, map, new_scale=None):
    if self.rgbMap != None:
      self.rgbMap.removeScalarMapListener(self)
    self.rgbMap       = map
    self.rgbMap.addScalarMapListener(self)
    self.colorControl = self.rgbMap.getControl()

    if new_scale != None:
      self.current = new_scale
    
    #- setupColorTables
    n_comps  = self.colorControl.getNumberOfComponents()
    n_colors = self.colorControl.getNumberOfColors()
      
    self.clr_table = []
    for i in xrange(n_comps):
      self.clr_table.append(jarray.zeros(n_colors, 'f'))
    self.clr_table = self.colorControl.initTableVis5D(self.clr_table)
      
    self.gry_table = []
    for i in xrange(n_comps):
      self.gry_table.append(jarray.zeros(n_colors, 'f'))
    self.gry_table = self.colorControl.initTableGreyWedge(self.gry_table)  
      
    self.igry_table = []
    for i in xrange(n_comps):
      self.igry_table.append(jarray.zeros(n_colors, 'f'))
    self.igry_table = self.colorControl.initTableGreyWedge(self.igry_table, 1)  
      
    self.n_comps = n_comps
    self.n_colors = n_colors
      
    if self.current == gray_cmd:
      self.colorControl.initGreyWedge()
      self.colorControl.changeControl(1)
    if self.current == igray_cmd:
      self.colorControl.initGreyWedge(1)
      self.colorControl.changeControl(1)
      
      
  def doMaskColorFunction(self, fnc, table):
    len = fnc.getLength()
    vals = fnc.getValues()
    new_vals = []
    if len == 258:
      for i in range(self.n_comps):
        ff = jarray.zeros(258, 'f')
        ff[0] = table[i][0]
        for j in range(256): ff[1+j] = table[i][j]
        ff[257]   = vals[i][257]
        new_vals.append(ff)
    elif len == 259:
      for i in range(self.n_comps):
        ff = jarray.zeros(259, 'f')
        ff[0] = table[i][0]
        for j in range(256): ff[1+j] = table[i][j]
        ff[257] = table[i][255]
        ff[258] = vals[i][258]
        new_vals.append(ff)
 
       
    elif len == 260:
      for i in range(self.n_comps):
        ff = jarray.zeros(260, 'f')
        ff[0] = table[i][0]
        for j in range(256): ff[1+j] = table[i][j]
        ff[257] = vals[i][257]
        ff[258] = vals[i][258]
        ff[259] = vals[i][259]
        new_vals.append(ff)
    elif len == 261:
      for i in range(self.n_comps):
        ff = jarray.zeros(261, 'f')
        ff[0] = table[i][0]
        for j in range(256): ff[1+j] = table[i][j]
        ff[257] = table[i][255]
        ff[258] = vals[i][258]
        ff[259] = vals[i][259]
        ff[260] = vals[i][260]
        new_vals.append(ff)

        
    elif len == 262:
      for i in range(self.n_comps):
        ff = jarray.zeros(262, 'f')
        ff[0] = table[i][0]
        for j in range(256): ff[1+j] = table[i][j]
        ff[257] = vals[i][257]
        ff[258] = vals[i][258]
        ff[259] = vals[i][259]
        ff[260] = vals[i][260]
        ff[261] = vals[i][261]
        new_vals.append(ff)
    elif len == 263:
      for i in range(self.n_comps):
        ff = jarray.zeros(263, 'f')
        ff[0] = table[i][0]
        for j in range(256): ff[1+j] = table[i][j]
        ff[257] = table[i][255]
        ff[258] = vals[i][258]
        ff[259] = vals[i][259]
        ff[260] = vals[i][260]
        ff[261] = vals[i][261]
        ff[262] = vals[i][262]
        new_vals.append(ff)
      
    fnc.setSamples(new_vals)
    
  def controlChanged(self, event):
    pass
    
  def mapChanged(self, event):
    if self.colorControl == None:
      self.colorControl = self.rgbMap.getControl()
    if self.colorControl == None:
      return
    elif self.initialized == 0:
      self.setup()

    if event.getId() == ScalarMapEvent.AUTO_SCALE:
      if self.invert == 1:
        range = self.rgbMap.getRange()
