import sys, jarray

sys.path.append('multispectral/Lib')
sys.path.append('multispectral/Lib/adde')
sys.path.append('multispectral/Lib/file')

from javax.swing import JPopupMenu, JFrame, JMenuBar, JMenu, JMenuItem, JTextField, JFileChooser, JLabel, JPanel, JButton, ToolTipManager
from javax.swing.border import LineBorder
from java.awt import GridLayout, BorderLayout, FlowLayout, Color, Dimension, Canvas, Toolkit, Component
from java.awt.event import InputEvent
from java.awt.datatransfer import DataFlavor
from java.lang import System, Object, Thread
from java.io import File

from visad import *
from visad.bom import HydraImageRendererJ3D, ImageRendererJ3D
import subs
import graph
from subs import *
from visad.python.JPythonMethods import *
from ui.colorbar import ColorBarWidget, ColorBar

import MultiSpectralDataSource
import cloudPhaseSource, cloudProductsSource, aerosolSource
from acViewer import acViewer
from cloudMaskSource import cloudMaskSource
from airsLevel2Source import airsLevel2Source
from cloudPhaseSource import mod06Source
from cloudProductsSource import mod07Source
from fireMaskDisplay import fireMaskDisplay
from mod05Source import mod05Source
from subsetImage import SubsetImage, Subset
from histostretch import linkHistLocation, dragHistLocation

from util import Helper
from html_hydra import HtmlBrowserWindow
from hframe import HFrame

import hydra_formulas
from hydra_formulas import distOnEarth
from hydra_formulas import radToBT, BTtoRad

#---------------------------------------
false = 0
true  = 1
lightWeightPopupEnabled = false
#---------------------------------------
version="v1.9.4"

JPopupMenu.setDefaultLightWeightPopupEnabled(lightWeightPopupEnabled)
ToolTipManager.sharedInstance().setLightWeightPopupEnabled(lightWeightPopupEnabled)

toolkit   = Toolkit.getDefaultToolkit()
clipboard = toolkit.getSystemClipboard()

import hydra_properties
hydra_properties.setProperties()

import hydra_globals
from hydra_globals import hydra_path
from hydra_globals import dataSelectDict

lastDirectory = None


class DataSelect(Object):

  def openURL_adde(self, event):
    import MultiSpectralDataSourceADDE
    self.ad = MultiSpectralDataSourceADDE.load(event.getActionCommand(), self.areaGui.getDay(), self.areaGui.getTime())

    self.enableFunctions()
    self.ad_clone = self.ad.clone()
    self.doStandalone()
    self.rebuildDisplay(self.ad_clone)
    self.resetCursor()

  def open_local_adde(self, url):
    import MultiSpectralDataSourceADDE
    self.ad = MultiSpectralDataSourceADDE.load(url, None, None)

    self.enableFunctions()
    self.ad_clone = self.ad.clone()
    self.doStandalone()
    self.rebuildDisplay(self.ad_clone)
    self.resetCursor()


  def openURL_html(self, url):
    filename = url.toString()
    prot = url.getProtocol()
    host = url.getHost()
    file = url.getFile()
    
    if host =="aqua.ssec.wisc.edu" or host=="terra.ssec.wisc.edu":
      filename = prot+"://"+host+"/cgi-bin/nph-dods"+file
    elif host == "g0dug03u.ecs.nasa.gov":
      filename = prot+"://"+host+"/daac-bin/dods/nph-dods"+file
    elif host == "thorpex-data.ssec.wisc.edu":
      filename = prot+"://"+host+"/cgi-bin/DODS/nph-dods"+file
    elif host == "bora7.ssec.wisc.edu":
      filename = prot+"://"+host+"/cgi-bin/nph-dods"+file
    elif host == "www.marsdb.it":
      filename = prot+"://"+host+"/cgi-bin/nph-dods"+file
      
    ad_l = MultiSpectralDataSource.load(filename)
   
    if ad_l.instrumentName == "MOD35":
       cMaskSrc=cloudMaskSource(ad_l)
    elif ad_l.instrumentName == "MOD06" or ad_l.instrumentName == "SeaDAS_L2PROD1":
       cldPhaseSrc = mod06Source(ad_l)
    elif ad_l.instrumentName == "MOD07":
       cldPrdSrc = mod07Source(ad_l)
    elif ad_l.instrumentName == "MOD05":
       cldPrdSrc = mod05Source(ad_l)
    elif ad_l.instrumentName == "MOD04":
       if self.loadListener != None:
         self.loadListener.setup()
         aerosolSource.AerosolData(ad_l)
       else:
         aerosolSource.makeMod04Display(ad_l)
    elif ad_l.instrumentName == "AIRSLevel2" or ad_l.instrumentName == "AIRSUWLevel2" or ad_l.instrumentName == "MODISUWLevel2" or ad_l.instrumentName == "MODISLevel2":
       aL2Src=airsLevel2Source(ad_l)
    else: # Level 1B Data
      self.ad_clone = ad_l.clone()
      self.ad = ad_l
      self.enableFunctions()
      self.doStandalone()
      self.rebuildDisplay(self.ad_clone)
      self.resetCursor()
    self.hbw.dispose()
  
  def fileChooseAction(self, event):
    global lastDirectory

    if lastDirectory != None:
      self.fileChoose.setCurrentDirectory(lastDirectory)

    if event.getActionCommand() == "Calipso":
      self.lidarSource = "Calipso"
      self.fileChoose.setDialogType(JFileChooser.OPEN_DIALOG)
      self.fileChoose.showOpenDialog(JFrame())

    if event.getActionCommand() == "CloudSat":
      self.cloudSatSource = "CloudSat"
      self.fileChoose.setDialogType(JFileChooser.OPEN_DIALOG)
      self.fileChoose.showOpenDialog(JFrame())

    if event.getActionCommand() == "AREA":
      self.areaSource = "AREA"
      self.fileChoose.setDialogType(JFileChooser.OPEN_DIALOG)
      self.fileChoose.showOpenDialog(JFrame())

    if event.getActionCommand() == "Local Data":
      self.areaSource = None
      self.lidarSource = None
      self.cloudSatSource = None
      self.fileChoose.setDialogType(JFileChooser.OPEN_DIALOG)
      self.fileChoose.showOpenDialog(JFrame())
      
    if event.getActionCommand() == "Remote Data":
      ##textFrame.setVisible(1)
      self.hbw = HtmlBrowserWindow(callback=self.openURL_html)
      self.hbw.show()
      
    if event.getActionCommand() == "GOES/MSG":
      from edu.wisc.ssec.mcidas.adde import GetAreaGUI_Hydra
      self.areaGui = GetAreaGUI_Hydra("get image", 0, 0, 0, actionPerformed=self.openURL_adde)
      self.areaGui.show()
  
    if event.getActionCommand() == "ApproveSelection":
      lastDirectory = self.fileChoose.getCurrentDirectory()
      filename = self.fileChoose.getSelectedFile().getPath()
      name = File(filename)
      name = name.getName()
      from java.net import URL
      if self.areaSource != None or name.startswith('AREA') or name.endswith('area'):
        self.open_local_adde('file://'+filename)
        self.areaSource = None
        return

      if self.lidarSource != None:
        import ldr_viewer
        if self.loadListener == None:
          table = ldr_viewer.fillMetadataTable('calipso')
          from ldr_viewer import ldr_viewer
          ldr_viewer(filename, None, None, metadata=table)
        else:
          ldr_viewer(filename, None, self.loadListener)
        self.lidarSource = None
        return

      if self.cloudSatSource != None:
        import ldr_viewer
        table = ldr_viewer.fillMetadataTable('cloudsat')
        from ldr_viewer import ldr_viewer
        ldr_viewer(filename, None, None, table)
        self.cloudSatSource = None
        return
        
      ad_l = MultiSpectralDataSource.load(filename)
      
      if ad_l.instrumentName == "MOD35":
         cMaskSrc=cloudMaskSource(ad_l)
      elif ad_l.instrumentName == "MOD14":
         fMaskDsp = fireMaskDisplay(ad_l)
      elif ad_l.instrumentName == "MOD06" or ad_l.instrumentName == "SeaDAS_L2PROD1":
         cldPhaseSrc = mod06Source(ad_l) 
      elif ad_l.instrumentName == "MOD07":
         cldPrdSrc = mod07Source(ad_l)
      elif ad_l.instrumentName == "MOD05":
         cldPrdSrc = mod05Source(ad_l)
      elif ad_l.instrumentName == "MOD04":
         aerosolSource.makeMod04Display(ad_l)   
      elif ad_l.instrumentName == "AIRSLevel2" or ad_l.instrumentName =="AIRSUWLevel2" or ad_l.instrumentName == "MODISUWLevel2" or ad_l.instrumentName == "MODISLevel2":
         aL2Src=airsLevel2Source(ad_l)
      else: # Level 1B Data
         self.ad_clone = ad_l.clone()
         self.ad = ad_l
         self.enableFunctions()
         self.rebuildDisplay(self.ad_clone)

  def pasteURL(event):
    transfer = clipboard.getContents(None)
    filename = transfer.getTransferData(DataFlavor.stringFlavor)
    filename = filename.replace(".html","")
    textField.setText(filename)
    
  def clearURL(event):
    textField.setText("")
   
  def openURL(event):
    filename = textField.getText()
    filename = filename.replace(".html","")
    ad_l = MultiSpectralDataSource.load(filename)
    self.enableFunctions()
    # load display based on the type of file
    if ad_l.instrumentName == "MOD35":
         cMaskSrc=cloudMaskSource(ad_l)
    elif ad_l.instrumentName == "MOD06" or ad_l.instrumentName == "SeaDAS_L2PROD1":
         cldPhaseSrc = mod06Source(ad_l) 
    elif ad_l.instrumentName == "MOD07":
         cldPrdSrc = mod07Source(ad_l)
    elif ad_l.instrumentName == "MOD04":
         aerosolSource.makeMod04Display(ad_clone)
    elif ad_l.instrumentName == "AIRSLevel2" or ad_l.instrumentName == "AIRSUWLevel2" or ad_l.instrumentName == "MODISUWLevel2" or ad_l.instrumentName == "MODISLevel2":
         aL2Src=airsLevel2Source(ad_l)
    else:
      self.ad_clone = ad_l.clone()
      self.ad = ad_l
      self.doStandalone()
      self.rebuildDisplay(ad_clone)
      self.resetCursor()
    
    
  def rebuildDisplay(self, ad):
    # clear display before creating a new one
    self.disp.removeAllReferences()
    self.disp.clearMaps()
                     
    obsElementInstrument, obsLineInstrument, obsChannelIndexInstrument, observationChannelsInstrument, observationsInstrument = ad.getRealTypes()
    observationInstrument = ad.imageRangeType
    obsLatitudeInstrument, obsLongitudeInstrument  = ad.getGeoRealTypes()
    
    latminmax, lonminmax = ad.getGeoDomain()
    
    londiff = lonminmax[1] - lonminmax[0]
    latdiff = latminmax[1] - latminmax[0]
    if londiff >= latdiff:
      lon_low = lonminmax[0]
      lon_hi  = lonminmax[1]
      lat_low = latminmax[0]
      lat_hi  = lat_low + londiff
    else:
      lat_low = latminmax[0]
      lat_hi  = latminmax[1]
      lon_low = lonminmax[0]
      lon_hi  = lon_low + latdiff
    
    if ad.instrumentName == "MODIS":
        channel_idx=32
    elif ad.instrumentName == "AVHRR":
        channel_idx=3
    elif ad.instrumentName == "MSG" or ad.instrumentName == "MSG_local":
        channel_idx=8
    elif ad.instrumentName == "AMSU_A":
        channel_idx=2
    elif ad.instrumentName == "AMSU_B":
        channel_idx=1
    else:
        channel_idx=int(ad.numObsChannels/3)
    
    sub = ad.getInitialSubset()
    self.sub = sub
    self.image = ad.getImageSegment(channel_idx, 0, sub)
  
    from visad import CommonUnit
    import mapProjections
    if ad.instrumentName == "MSG":
      corners = [[-80, 80, -80, 80], [60, 60, -60, -60]]
    elif ad.instrumentName == "GOES-E":
      corners = [[-150, 10, -150, 10], [60, 60, -60, -60]]
    elif ad.instrumentName == "GOES-W":
      corners = [[-150, 10, -150, 10], [60, 60, -60, -60]]
    elif ad.instrumentName == "GOES-10":
      corners = [[-150, 10, -150, 10], [60, 60, -60, -60]]
    else:
      corners = mapProjections.getValidCorners(ad.gs, [[-180,180], [-90, 90]])
    
    minLon = min(corners[0])
    minLat = min(corners[1])
    maxLon = max(corners[0])
    maxLat = max(corners[1])
  
    lon_low = minLon
    lat_low = minLat
    lon_hi  = maxLon
    lat_hi  = maxLat
    
    displayLatType   = DisplayRealType("ProjectionLat_%d" % hydra_globals.dtuple_cnt, true, lat_low, lat_hi, 0.0, CommonUnit.degree)
    displayLonType   = DisplayRealType("ProjectionLon_%d" % hydra_globals.dtuple_cnt, true, lon_low, lon_hi, 0.0, CommonUnit.degree)
    displayAltType   = DisplayRealType("ProjectionAlt_%d" % hydra_globals.dtuple_cnt, true, -1.0, 1.0, 0.0, None)
    
    from util import LambertEA, SatProjection, MapProjectionAdapter
    if ad.instrumentName == "MSG" or ad.instrumentName == "GOES-E" or ad.instrumentName == "GOES-W" or ad.instrumentName == "GOES-10":
      adaptProj = MapProjectionAdapter(SatProjection(ad.imageField, true, ad.cs))
    else:
      adaptProj = MapProjectionAdapter(LambertEA(corners))
  
    displayTupleType = DisplayTupleType([displayLonType, displayLatType, displayAltType], adaptProj)
  
    maps = []
    maps.append(ScalarMap(obsLongitudeInstrument, displayLonType))
    maps.append(ScalarMap(obsLatitudeInstrument, displayLatType))
    maps.append(ScalarMap(observationInstrument, Display.RGB))
    #- for rubber band zoomer
    maps.append(ScalarMap(RealType.XAxis, Display.XAxis))
    maps.append(ScalarMap(RealType.YAxis, Display.YAxis))
    maps[3].setRange(-1, 1)
    maps[4].setRange(-1, 1)
  
    self.disp.addMaps(maps)

    self.disp.disableAction()
  
    maps[0].setRange(lon_low, lon_hi)
    maps[1].setRange(lat_low, lat_hi)  
    
    imageRef=subs.addData('a',self.image,self.disp,renderer=ImageRendererJ3D(), zlayer = -0.1)
    subs.addData('b',self.poliMap,self.disp,constantMaps=makeColorMap(hydra_properties.foreground_color))
    
    si = SubsetImage(self.image,self.disp,maps[0],maps[1],self.changeSubset,self.subsetRef,color="green",ds_cs=adaptProj)   
    new_scale = None
    if ad.instrumentName == "MODIS_500m" or ad.instrumentName == "MODIS_250m":
      new_scale = 'Gray'
    self.colorMenu.changeColorScalarMap(maps[2], new_scale)
    hydra_globals.dtuple_cnt += 1
    
    # add labels to the right of the color bar
    colorBar = ColorBarWidget(maps[2],hydra_properties.background_color, hydra_properties.foreground_color)
    colorBar.setPreferredSize(Dimension(20,400))
  
    self.maps = maps
    self.disp.enableAction()
  
        
  def changeSubset(self,a,b):
    try: #- incase a,b too big
      x0 = int(min(a))
      y0 = int(min(b))
      x1 = int(max(a))
      y1 = int(max(b))
    except:
      return
    
    #-- image to segment coordinates
    xy0 = self.ad_clone.subsetDomain.gridToValue([[x0],[y0]])
    xy1 = self.ad_clone.subsetDomain.gridToValue([[x1],[y1]])
    
    try: #- incase coords can't be converted to float
      x0  = int(xy0[0][0])
      y0  = int(xy0[1][0])
      x1  = int(xy1[0][0])
      y1  = int(xy1[1][0])
    except:
      return
    
    if (((x1 - x0) > 10) or ((y1 - y0) > 10)):
      ss  = Subset(x0,y0,x1,y1)
      self.sub_last = ss
      self.sub = ss
      self.doStandalone()
      if self.ac != None:
        for i in xrange(len(dataSelectDict[self])):
          (dataSelectDict[self])[i].changeSubset(ss)
          Thread.sleep(1500)
        ##self.ac.changeSubset(ss)
      else:
        print "Start Multispectral Viewer First"
      self.resetCursor()
    
  def start_acViewer(self,event):
    self.doStandalone()

    if self.loadListener != None:
      self.loadListener.setup()

    if self.ad != None:
      ##sub = self.ad.getInitialSubset()
      
      if self.ad == self.ad_last:
        ##self.ac = acViewer(self.ad.clone(), sub, self.ext_display, self.idv_displayMaster)
        self.ac = acViewer(self.ad.clone(), self.sub, self.ext_display, self.idv_displayMaster, self)
        (dataSelectDict[self]).append(self.ac)
      else:
        ##self.ac = acViewer(self.ad, sub, self.ext_display, self.idv_displayMaster)
        self.ac = acViewer(self.ad, self.sub, self.ext_display, self.idv_displayMaster, self)
        if len(dataSelectDict[self]) == 0:
          (dataSelectDict[self]).append(self.ac)
        else:
          dataSelectDict[self] = []
          (dataSelectDict[self]).append(self.ac)

      #(dataSelectDict[self]).append(self.ac)
     

      self.ad_last  = self.ad

      if self.loadListener != None:
        self.ac.idv = self.loadListener
        self.loadListener.load(self.ac.observationsInstrument, self.ac.refToImage, None, 'h img', 'hydra image')
        self.ac.rgbMap = self.hydraDisplayable.getColorMap()
        """
        import multiSpectralControl
        from multiSpectralControl import MultiSpectralControl
        msc = MultiSpectralControl(self.ad)
        self.loadListener.load(msc.rgbRealType, msc.dataReference, msc.positionRef, 'h img', 'hydra image')
        msc.setColorMap(self.hydraDisplayable.getColorMap())
        msc.setSelectRangeMap(self.hydraDisplayable.getAnimationMap())
        """
    else:
      print "Please open a Local/Remote file"

    hydra_globals.mcViewer = self.ac
    self.resetCursor()
      
  def setColorScale(self,event):
    dragHistLocation(self.maps[2], self.image)
    
  def captureDisplay(self,event):
    Helper.saveDisplay(self.disp, JFrame().getContentPane())
    
  def exit(self,event):
    System.exit(0)

  def new(self,event):
    bigFrame  = HFrame("Hydra  "+"(Version: "+version+")", windowClosing=exit)
    data_selector = DataSelect(frame=bigFrame)
    
  def getMenuBar(self):
    return self.menuBar

  def getToolBar(self):
    return self.toolbar

  def getDisplay(self):
    return self.disp

  def getComponent(self):
    "@sig public java.awt.Component getComponent()"
    return self.component

  def doStandalone(self):
    if self.standalone == true:
      self.frame.toFront()
      self.frame.setCursor(3)

  def resetCursor(self):
    if self.standalone == true:
      self.frame.setCursor(0)

  def enableFunctions(self):
    self.colorRangeItem.setEnabled(1)
    self.colorMenu.setEnabled(1)
    if self.standalone == true:
      self.mcItem.setEnabled(1)
    self.mcItem.setEnabled(1)

  def addLoadListener(self, obj):
    self.loadListener = obj
  
  def __init__(self, frame=None):
    self.mainPanel = None
    self.ac = None
    self.ad_last = None
    self.subsetRef = []
    self.subsetRef.append(None)
    self.frame = frame
    self.component = None

    self.loadListener = None
    self.ext_display = None
    self.hydraDisplayable = None
    self.idv_displayMaster = None
    self.idv_viewManager = None
    self.idv_displayInfo = None
    self.lidarSource = None
    self.cloudSatSource = None
    self.areaSource = None
    self.sub_last = None
    self.sub = None
    dataSelectDict[self] = []

    if self.frame != None:
      self.standalone = true
    else:
      self.standalone = false

    #-- setup initial gui

    self.menuBar   = JMenuBar()

    fileMenu = JMenu("File")

    self.menuBar.add(fileMenu)
    exitItem = JMenuItem("Exit", actionPerformed=self.exit)
    exitItem.setEnabled(1)

    newItem = JMenuItem('New', actionPerformed=self.new)
    newItem.setEnabled(1)
    fileMenu.add(newItem)
    fileMenu.add(exitItem)

    # Populate the Load menu
    dataMenu = JMenu("Load")
    localItem  = JMenuItem("Local Data", actionPerformed=self.fileChooseAction)
    localItem.setEnabled(1)
    remoteItem = JMenuItem("Remote Data", actionPerformed=self.fileChooseAction)
    remoteItem.setEnabled(1)
    dataMenu.add(localItem)
    dataMenu.add(remoteItem)
    addeItem = JMenuItem("GOES/MSG", actionPerformed=self.fileChooseAction)
    addeItem.setEnabled(1)
    dataMenu.add(addeItem)
    lidarItem = JMenuItem("Calipso", actionPerformed=self.fileChooseAction)
    lidarItem.setEnabled(1)
    dataMenu.add(lidarItem)
    lidarItem = JMenuItem("CloudSat", actionPerformed=self.fileChooseAction)
    lidarItem.setEnabled(1)
    dataMenu.add(lidarItem)
    lidarItem = JMenuItem("AREA", actionPerformed=self.fileChooseAction)
    lidarItem.setEnabled(1)
    dataMenu.add(lidarItem)
    self.menuBar.add(dataMenu)
  
    # Populate the Settings menu
    settingsMenu = JMenu("Settings")
    self.colorRangeItem = JMenuItem("Set Color Range", actionPerformed=self.setColorScale)
    self.colorRangeItem.setEnabled(0)
    settingsMenu.add(self.colorRangeItem)
    self.menuBar.add(settingsMenu)

    # Populate the Tools menu
    toolMenu = JMenu("Tools")
    item2 = JMenuItem("Capture Display", actionPerformed=self.captureDisplay)
    item2.setEnabled(1)
    toolMenu.add(item2)
    self.menuBar.add(toolMenu)
  
    # Populate the Color menu
    from ColorScaleSelect import ColorScaleSelect
    self.colorMenu = ColorScaleSelect("Set Color Scale", None)
    self.colorMenu.setEnabled(0)
    settingsMenu.add(self.colorMenu)
    self.menuBar.add(settingsMenu)
  
    # Populate the Start menu
    ##if self.standalone == true:
    startMenu = JMenu("Start")
    self.menuBar.add(startMenu)
    self.mcItem = JMenuItem("Multi-Channel Viewer", actionPerformed=self.start_acViewer)
    self.mcItem.setEnabled(0)
    startMenu.add(self.mcItem)
  
    self.fileChoose = JFileChooser(actionPerformed=self.fileChooseAction)

    # create the maps display
    maps = makeMaps(RealType.Longitude, 'x', RealType.Latitude, 'y')
    self.disp = makeDisplay(maps)
    mode = self.disp.getGraphicsModeControl()
    mode.setCurvedSize(1)
    dsp_rdr = self.disp.getDisplayRenderer()
    dsp_rdr.setBackgroundColor(hydra_properties.background_color)
    dsp_rdr.setForegroundColor(hydra_properties.foreground_color)
  
    from DisplayViewSelect import DisplayViewSelectToolBar, RubberBandSubset
    self.toolbar = DisplayViewSelectToolBar([self.disp])
    self.toolbar.addHAction(RubberBandSubset(self.toolbar.listeners, self.subsetRef, icon=hydra_path+'/ui/icons/subset12.jpg', tooltipText="Subset Image"))

    if self.standalone == false:
      topPanel = JPanel(GridLayout(2,1))
      topPanel.add(self.menuBar)
      topPanel.add(self.toolbar)
      self.component = JPanel(BorderLayout())
      self.component.add("North", topPanel)
      self.component.add("Center", self.disp.getComponent())
    else:
      self.frame.setJMenuBar(self.menuBar)
      self.component = self.frame.getContentPane()
      self.component.setLayout(BorderLayout())
      self.component.add("Center", self.disp.getComponent())
      self.component.add("North", self.toolbar)
      self.frame.pack()
      self.frame.setSize(400,400)
      self.frame.show()

  
    self.poliMap=load(hydra_path+'/data/outlsupw')
    subs.addData('supw',self.poliMap,self.disp, constantMaps=makeColorMap(hydra_properties.foreground_color))



def exit(event):
  System.exit(0)

if __name__ == '__main__':
  bigFrame  = HFrame("Hydra  "+"(Version: "+version+")", windowClosing=exit)
  data_selector = DataSelect(frame=bigFrame)
