import sys

from javax.swing import JPopupMenu, JFrame, JMenuBar, JMenu, JMenuItem, JTextField, JFileChooser, JLabel, JPanel, JSplitPane, JButton
from javax.swing.border import LineBorder
from java.awt import GridLayout, BorderLayout, FlowLayout, Color, Dimension, Canvas, Toolkit 
from java.awt.event import InputEvent
from java.awt.datatransfer import DataFlavor
from java.lang import System, Runtime

from visad import *
from visad.bom import HydraImageRendererJ3D
import mapProjections
import subs
import graph
from subs import *
from visad.python.JPythonMethods import *
from ui.colorbar import ColorBarWidget, ColorBar

import MultiSpectralDataSource
import jarray
from hframe import HFrame
from subsetImage import SubsetImage, Subset
from histostretch import linkHistLocation, dragHistLocation

from util import Helper
from util import LambertEA, SatProjection, MapProjectionAdapter
from html_hydra import HtmlBrowserWindow

import hydra_properties
hydra_properties.setProperties()

false = 0
true  = 1

global adaptProj
import hydra_globals
from hydra_globals import hydra_path

runtime=Runtime.getRuntime()
subsetRef=[]
subsetRef.append(None)

class mod06Source:
  def __init__(self, displayData, subset=None):
    self.displayData = displayData
    self.currentLine = 0
    self.currentElem = 0
    self.dspInit = false
    self.standalone = true
    self.currentSubset = None
    self.ovlyClr_fnc = None

     # open a new window if opening Level 2 Products
    self.frame  = HFrame("Hydra MOD06 Level 2 Products",windowClosing=self.windowClose)
    self.framePane = self.frame.getContentPane()
     
    menuBar   = JMenuBar()
    self.frame.setJMenuBar(menuBar)
     
    varMenu = JMenu("Variables")
     
    varsToAdd=displayData.imageRangeTypeDict.keys()
    for i in xrange(displayData.numObsDataSets):
        varMenuItem=JMenuItem(varsToAdd[i],actionPerformed=self.changeVariable)
        varMenuItem.setEnabled(1)
        varMenu.add(varMenuItem)
     
    menuBar.add(varMenu)
     
    toolMenu = JMenu("Tools")
    toolItem = JMenuItem("Capture Display", actionPerformed=self.captureDisplay)
    toolItem.setEnabled(1)
    toolMenu.add(toolItem)
    menuBar.add(toolMenu)
    self.toolMenu = toolMenu
    self.varMenu = varMenu

    self.obsLatitudeInstrument, self.obsLongitudeInstrument  = self.displayData.getGeoRealTypes()
    self.observationsInstrument = self.displayData.imageRangeType
    self.mcv = None
    self.ratio = None

    if hydra_globals.mcViewer != None:
      #- overlay on existing MultiChannelViewer image display ---
      self.overlaySetup()
    else:
      self.standaloneSetup()
     
      #  phase toggle button 
    self.togText  = ["Overlay on MODIS Image", "Toggle Off Overlay"]
    self.togIndex = 1
    self.lastTogIndex = 0
    self.phaseToggleButton = JButton(self.togText[self.togIndex], actionPerformed=self.toggleCloudPhase)
    if self.mcv == None:
      self.phaseToggleButton.setEnabled(0)
      self.phaseToggleButton.setText("Start Multi-Channel Viewer to Overlay")
    else:
      self.phaseToggleButton.setEnabled(1)

    self.framePane.add("South",self.phaseToggleButton)
     
  def overlaySetup(self):
    self.standalone = false
    self.framePane.setLayout(BorderLayout())
    self.currentSubset = hydra_globals.mcViewer.currentSubset
    self.mcv = hydra_globals.mcViewer
    self.mcv.addUpdateListener(self)
    self.imageDisplay = self.mcv.getImageDisplay()
    self.rgbMap = ScalarMap(self.observationsInstrument, Display.RGBA)
    self.imageDisplay.addMap(self.rgbMap)
    self.ratio = self.mcv.ad.numObsElements/self.displayData.numObsElements
      
    if self.displayData.paramName == "Cloud Phase":
      colors = [[],[],[],[]]
      fsmpls = jarray.zeros(12, 'f')
      fsmpls[0] = 0
      colors[0].append(0.0), colors[1].append(0.0), colors[2].append(0.0), colors[3].append(0.0)
      fsmpls[1] = 0.16
      colors[0].append(0.0), colors[1].append(0.0), colors[2].append(0.0), colors[3].append(0.0)
      fsmpls[2] = 0.17
      colors[0].append(0.0), colors[1].append(0.0), colors[2].append(1.0), colors[3].append(1.0)
      fsmpls[3] = 0.33
      colors[0].append(0.0), colors[1].append(0.0), colors[2].append(1.0), colors[3].append(1.0)
      fsmpls[4] = 0.34
      colors[0].append(1.0), colors[1].append(0.5), colors[2].append(0.5), colors[3].append(1.0)
      fsmpls[5] = 0.49
      colors[0].append(1.0), colors[1].append(0.5), colors[2].append(0.5), colors[3].append(1.0)
      fsmpls[6] = 0.50
      colors[0].append(1.0), colors[1].append(1.0), colors[2].append(0.0), colors[3].append(1.0)
      fsmpls[7] = 0.67
      colors[0].append(1.0), colors[1].append(1.0), colors[2].append(0.0), colors[3].append(1.0)
      fsmpls[8] = 0.68
      colors[0].append(0.5), colors[1].append(1.0), colors[2].append(0.0), colors[3].append(1.0)
      fsmpls[9] = 0.85
      colors[0].append(0.5), colors[1].append(1.0), colors[2].append(0.0), colors[3].append(1.0)
      fsmpls[10] = 0.86
      colors[0].append(0.0), colors[1].append(0.0), colors[2].append(0.0), colors[3].append(1.0)
      fsmpls[11] = 1.00
      colors[0].append(0.0), colors[1].append(0.0), colors[2].append(0.0), colors[3].append(1.0)
    
      clr_set = Gridded1DSet(RealType.Generic, [fsmpls], len(fsmpls))
      clr_fnc = FlatField(FunctionType(RealType.Generic, RealTupleType([makeType("cR"), makeType("cG"), makeType("cB"), makeType("cA")])), clr_set)
      clr_fnc.setSamples(colors)
    
      cc = self.rgbMap.getControl()
      cc.setFunction(clr_fnc)
      self.ovlyClr_fnc = clr_fnc
      self.rgbMap.setRange(0,5)  #- to make sure the phase values come up the right color. 
    
      # add labels to the right of the color bar
      colorBar = ColorBar(self.rgbMap,false)
      colorBar.setPreferredSize(Dimension(20,400))
      colorBar.setLayout(GridLayout(1,2,0,0))
      cbLabelPanel=JPanel(GridLayout(6,1,0,0))
      cbLabelPanel.setPreferredSize(Dimension(20,400))

      self.labelsToAdd = ["Clear","Water","Ice","Mixed","Uncertain","Missing Data"]
      for i in xrange(len(self.labelsToAdd)):
        cbLabelPanel.add(JLabel(self.labelsToAdd[i]))
      colorBar.add(cbLabelPanel)    
      self.cbLabelPanel = cbLabelPanel

    else:
      (self.imageDisplay.getGraphicsModeControl()).setMissingTransparent(true)
      rng = self.displayData.getObsValidRange()
      self.rgbMap.setRange(rng[0], rng[1])
      colorBar = ColorBarWidget(self.rgbMap,hydra_properties.background_color, hydra_properties.foreground_color)
      colorBar.setPreferredSize(Dimension(20,400))


    self.img = self.getImageField()
    self.imageRef = subs.addData('imageRef', self.img, self.imageDisplay, constantMaps=[ConstantMap(-0.2,self.mcv.displayAltType)], renderer=HydraImageRendererJ3D())
    self.currentSubset = self.mcv.currentSubset

    self.framePane.add("Center", colorBar)
    self.colorBar = colorBar
    self.toolMenu.setVisible(1)
    self.varMenu.setVisible(1)

    self.frame.pack()
    self.frame.setSize(300,500)
    self.frame.show()
             
  def standaloneSetup(self):
    self.splitPane=JSplitPane(JSplitPane.HORIZONTAL_SPLIT)

    self.rebuildDisplay()

    self.map=load(hydra_path+'/data/outlsupw')
    self.mapRef = subs.addData('supw',self.map,self.imageDisplay,constantMaps=makeColorMap(hydra_properties.foreground_color))

    framePane = self.frame.getContentPane()
    framePane.setLayout(BorderLayout())
    self.splitPane.add(self.imageDisplay.getComponent())
    self.splitPane.setContinuousLayout(1)
    self.splitPane.setOneTouchExpandable(1)
    self.splitPane.setDividerSize(12)
    framePane.add("Center",self.splitPane)
                                                                                                                                           
    from DisplayViewSelect import DisplayViewSelectToolBar, RubberBandSubset
    toolbar = DisplayViewSelectToolBar([self.imageDisplay])
    framePane.add("North", toolbar)
                                                                                                                                           
    self.toolbar=toolbar
    self.text = TextType.Generic


    self.framePane = framePane
    self.frame.pack()
    self.frame.setSize(650,500)
    self.frame.show()


  def rebuildDisplay(self):
     # clear display before creating a new one
     if (self.dspInit == true or self.standalone == false):
       self.imageDisplay.removeReference(self.imageRef)
       self.imageDisplay.removeMap(self.rgbMap)

     if self.standalone == true:
       # get rid of the right pane (colorbar) if it exists as it needs to be recreated
       testForColorbar=self.splitPane.getLeftComponent()
       if testForColorbar != None:
         self.splitPane.remove(testForColorbar)
     else:
       self.framePane.remove(self.colorBar)

     if self.displayData.numObsElements < 512:
        img = self.getImageField()
     else:
        img = self.getImageField(Subset(None,None,None,None,step=10))

     self.observationsInstrument = self.displayData.imageRangeType

     ir=HydraImageRendererJ3D()

     if (self.dspInit == false and self.standalone == true):
        
       self.obsLatitudeInstrument, self.obsLongitudeInstrument  = self.displayData.getGeoRealTypes()
       corners = mapProjections.getValidCorners(self.displayData.gs, [[-180,180], [-90, 90]])
     
       minLon = min(corners[0])
       minLat = min(corners[1])
       maxLon = max(corners[0])
       maxLat = max(corners[1])

       displayLatType   = DisplayRealType("ProjectionLat_%d" % hydra_globals.dtuple_cnt, true, minLat, maxLat, 0.0, CommonUnit.degree)
       displayLonType   = DisplayRealType("ProjectionLon_%d" % hydra_globals.dtuple_cnt, true, minLon, maxLon, 0.0, CommonUnit.degree)
       displayAltType   = DisplayRealType("ProjectionAlt_%d" % hydra_globals.dtuple_cnt, true, -1.0, 1.0, 0.0, None)

       adaptProj = MapProjectionAdapter(LambertEA(corners))
     
       self.ds_cs = adaptProj
     
       displayTupleType = DisplayTupleType([displayLonType, displayLatType, displayAltType], adaptProj)
       self.displayAltType = displayAltType

       if self.displayData.paramName == "Cloud Phase":   
         self.maps = subs.makeMaps(self.obsLongitudeInstrument,'x', self.obsLatitudeInstrument, 'y', self.observationsInstrument, 'rgba')
       else:
         self.maps = subs.makeMaps(self.obsLongitudeInstrument,'x', self.obsLatitudeInstrument, 'y', self.observationsInstrument, 'rgba')     
       
       #- replace first two with user-defined display types
       self.maps[0] = ScalarMap(self.obsLongitudeInstrument, displayLonType)
       self.maps[1] = ScalarMap(self.obsLatitudeInstrument, displayLatType)
       #- for rubber band zoomer
       self.maps.append(ScalarMap(RealType.XAxis, Display.XAxis))
       self.maps.append(ScalarMap(RealType.YAxis, Display.YAxis))
       self.maps[3].setRange(-1, 1)
       self.maps[4].setRange(-1, 1)

       self.imageDisplay = makeDisplay(self.maps)

       # get the dimensions
       dimInstrumentDomain = img.getDomainSet()
       self.dimInstrument = dimInstrumentDomain.getLengths()
  
       self.imageDisplay.disableAction()
               
       self.maps[0].setRange(minLon, maxLon)
       self.maps[1].setRange(minLat, maxLat) 
       self.rgbMap = self.maps[2]
       self.dspInit = true
       ir.setMissingColor(0,0,0) # set missing to black (Liam G request)

     else:
       self.rgbMap = ScalarMap(self.observationsInstrument, Display.RGBA)
       rng = self.displayData.getObsValidRange()
       self.rgbMap.setRange(rng[0], rng[1])
       self.imageDisplay.addMap(self.rgbMap)
                
     self.imageRef=subs.addData('a', img, self.imageDisplay, renderer=ir, zlayer = -0.1)

     ###subs.addData('b',self.map,self.imageDisplay,constantMaps=makeColorMap(hydra_properties.foreground_color))
     

     if self.displayData.paramName == "Cloud Phase":
       # create the cloud phase colorbar
       colors = [[],[],[],[]]
       fsmpls = jarray.zeros(12, 'f')
       fsmpls[0] = 0
       colors[0].append(0.0), colors[1].append(0.0), colors[2].append(0.0), colors[3].append(1.0)
       fsmpls[1] = 0.16
       colors[0].append(0.0), colors[1].append(0.0), colors[2].append(0.0), colors[3].append(1.0)
       fsmpls[2] = 0.17
       colors[0].append(0.0), colors[1].append(0.0), colors[2].append(1.0), colors[3].append(1.0)
       fsmpls[3] = 0.33
       colors[0].append(0.0), colors[1].append(0.0), colors[2].append(1.0), colors[3].append(1.0)
       fsmpls[4] = 0.34
       colors[0].append(1.0), colors[1].append(0.5), colors[2].append(0.5), colors[3].append(1.0)
       fsmpls[5] = 0.49
       colors[0].append(1.0), colors[1].append(0.5), colors[2].append(0.5), colors[3].append(1.0)
       fsmpls[6] = 0.50
       colors[0].append(1.0), colors[1].append(1.0), colors[2].append(0.0), colors[3].append(1.0)
       fsmpls[7] = 0.67
       colors[0].append(1.0), colors[1].append(1.0), colors[2].append(0.0), colors[3].append(1.0)
       fsmpls[8] = 0.68
       colors[0].append(0.5), colors[1].append(1.0), colors[2].append(0.0), colors[3].append(1.0)
       fsmpls[9] = 0.85
       colors[0].append(0.5), colors[1].append(1.0), colors[2].append(0.0), colors[3].append(1.0)
       fsmpls[10] = 0.86
       colors[0].append(0.0), colors[1].append(0.0), colors[2].append(0.0), colors[3].append(1.0)
       fsmpls[11] = 1.00
       colors[0].append(0.0), colors[1].append(0.0), colors[2].append(0.0), colors[3].append(1.0)
       clr_set = Gridded1DSet(RealType.Generic, [fsmpls], len(fsmpls))
                                                                                                                                          
       clr_fnc = FlatField(FunctionType(RealType.Generic, RealTupleType([makeType("cR"), makeType("cG"), makeType("cB"), makeType("cA")])), clr_set)
       clr_fnc.setSamples(colors)
       cc = self.rgbMap.getControl()
       cc.setFunction(clr_fnc)
        
       self.rgbMap.setRange(0,5)  #- to make sure the phase values come up the right color. 
                       
       # add labels to the right of the color bar
       colorBar = ColorBar(self.rgbMap,false)
       colorBar.setPreferredSize(Dimension(20,400))
       colorBar.setLayout(GridLayout(1,2,0,0))
       cbLabelPanel=JPanel(GridLayout(6,1,0,0))
       cbLabelPanel.setPreferredSize(Dimension(20,400))

       self.labelsToAdd = ["Clear","Water","Ice","Mixed","Uncertain","Missing Data"]
       for i in xrange(len(self.labelsToAdd)):
         cbLabelPanel.add(JLabel(self.labelsToAdd[i]))
         
       colorBar.add(cbLabelPanel)
     else:
       colorBar = ColorBarWidget(self.rgbMap,hydra_properties.background_color, hydra_properties.foreground_color)
       colorBar.setPreferredSize(Dimension(20,400))
       self.framePane.add(colorBar)
       self.colorBar = colorBar
     
     if self.mcv != None:
       self.frame.pack()
     else:  
       self.splitPane.add(colorBar)     
       self.splitPane.setDividerLocation(250)
     
     hydra_globals.dtuple_cnt += 1
     self.img = img
          
     self.imageDisplay.enableAction()
     
  def windowClose(self,event):
    if self.mcv != None:
      self.imageDisplay.removeReference(self.imageRef)
      self.imageDisplay.removeMap(self.rgbMap)
      self.mcv.removeUpdateListener(self)
    self.frame.dispose()
    runtime.gc()

  def captureDisplay(self,event):
    Helper.saveDisplay(self.imageDisplay, JFrame().getContentPane())
  
  def changeVariable(self,event):
    self.displayData.changeParameter(event.source.text)
    event.source.setSelected(1)
    
    # reinitialize observations and display
    self.displayData.observationField = None
    self.ovlyClr_fnc = None
    self.rebuildDisplay()
     
  def toggleCloudPhase(self, event):
    if self.mcv != None:
      if (self.togIndex == 0):
        self.imageDisplay.toggle(self.imageRef, 1)
      if (self.togIndex == 1):
        self.imageDisplay.toggle(self.imageRef, 0)
      self.phaseToggleButton.setText(self.togText[self.lastTogIndex])
      tmp = self.togIndex
      self.togIndex = self.lastTogIndex
      self.lastTogIndex = tmp
   
  def updateDisplay(self, subset=None):
    if subset != None:
      self.img = self.getImageField(subset)

    self.imageDisplay.disableAction()
    self.imageDisplay.removeReference(self.imageRef)
    self.imageDisplay.removeMap(self.rgbMap)

    self.rgbMap = ScalarMap(self.observationsInstrument, Display.RGBA)
    rng = self.displayData.getObsValidRange()
    if rng != None:
      self.rgbMap.setRange(rng[0], rng[1])
    self.imageDisplay.addMap(self.rgbMap)

    if self.ovlyClr_fnc != None:
      self.cc = self.rgbMap.getControl()
      self.cc.setFunction(self.ovlyClr_fnc)
      self.rgbMap.setRange(0, 5)

    self.imageRef = subs.addData('imageRef', self.img, self.imageDisplay, constantMaps=[ConstantMap(-0.2, self.mcv.displayAltType)], renderer=HydraImageRendererJ3D())
    self.imageDisplay.enableAction()

  def getImageField(self, subset=None):
    if subset != None:
      self.currentSubset = subset
    if self.displayData.unnavigated == false:
      cs = self.currentSubset
      self.ratio = self.mcv.ad.numObsElements/self.displayData.numObsElements
      step = cs.step/self.ratio
      if (step < 1): step = 1
      subset = Subset(cs.start_x, cs.start_y, cs.end_x, cs.end_y, step)
      subset.scale(1.0/self.ratio)
      return self.displayData.getLevel2ImageSegment(0,subset)
    else:
      mcvImg = self.mcv.refToImage.getData()
      fld = self.displayData.getLevel2ImageSegment(0,self.currentSubset)
      new_mtype = FunctionType(mcvImg.getType().getDomain(), fld.getType().getRange())
      new_field = FlatField(new_mtype, mcvImg.getDomainSet())
      new_field.setSamples(Helper.unpackFloats(fld,true), false)
      return new_field

"""
  def getImageColorWidget(self):
    # create the cloud phase colorbar with transparency
    colors = [[],[],[],[]]
    fsmpls = jarray.zeros(12, 'f')
    fsmpls[0] = 0
    colors[0].append(0.0), colors[1].append(0.0), colors[2].append(0.0), colors[3].append(0.0)
    fsmpls[1] = 0.16
    colors[0].append(0.0), colors[1].append(0.0), colors[2].append(0.0), colors[3].append(0.0)
    fsmpls[2] = 0.17
    colors[0].append(0.0), colors[1].append(0.0), colors[2].append(1.0), colors[3].append(1.0)
    fsmpls[3] = 0.33
    colors[0].append(0.0), colors[1].append(0.0), colors[2].append(1.0), colors[3].append(1.0)
    fsmpls[4] = 0.34
    colors[0].append(1.0), colors[1].append(0.5), colors[2].append(0.5), colors[3].append(1.0)
    fsmpls[5] = 0.49
    colors[0].append(1.0), colors[1].append(0.5), colors[2].append(0.5), colors[3].append(1.0)
    fsmpls[6] = 0.50
    colors[0].append(1.0), colors[1].append(1.0), colors[2].append(0.0), colors[3].append(1.0)
    fsmpls[7] = 0.67
    colors[0].append(1.0), colors[1].append(1.0), colors[2].append(0.0), colors[3].append(1.0)
    fsmpls[8] = 0.68
    colors[0].append(0.5), colors[1].append(1.0), colors[2].append(0.0), colors[3].append(1.0)
    fsmpls[9] = 0.85
    colors[0].append(0.5), colors[1].append(1.0), colors[2].append(0.0), colors[3].append(1.0)
    fsmpls[10] = 0.86
    colors[0].append(0.0), colors[1].append(0.0), colors[2].append(0.0), colors[3].append(1.0)
    fsmpls[11] = 1.00
    colors[0].append(0.0), colors[1].append(0.0), colors[2].append(0.0), colors[3].append(1.0)
                                                                                                                                           
    clr_set = Gridded1DSet(RealType.Generic, [fsmpls], len(fsmpls))
    clr_fnc = FlatField(FunctionType(RealType.Generic, RealTupleType([makeType("cR"), makeType("cG"), makeType("cB"), makeType("cA")])), clr_set)
    clr_fnc.setSamples(colors)
                                                                                                                                           
    self.rgbMap = ScalarMap(self.observationsInstrument, Display.RGBA)
    self.imageDisplay.addMap(self.rgbMap)
    cc = self.rgbMap.getControl()
    cc.setFunction(clr_fnc)
    self.ovlyClr_fnc = clr_fnc
    self.rgbMap.setRange(0,5)  #- to make sure the phase values come up the right color.
                                                                                                                                           
    # add labels to the right of the color bar
    colorBar = ColorBar(self.rgbMap,false)
    colorBar.setPreferredSize(Dimension(20,400))
    colorBar.setLayout(GridLayout(1,2,0,0))
    cbLabelPanel=JPanel(GridLayout(6,1,0,0))
    cbLabelPanel.setPreferredSize(Dimension(20,400))
                                                                                                                                           
    self.labelsToAdd = ["Clear","Water","Ice","Mixed","Uncertain","Missing Data"]
    for i in xrange(len(self.labelsToAdd)):
      cbLabelPanel.add(JLabel(self.labelsToAdd[i]))
    colorBar.add(cbLabelPanel)
"""
