from visad import CellImpl, RealTuple, RealTupleType, RealType, ConstantMap, DisplayListener, DisplayEvent, Display, Real
from visad import DataReferenceImpl
from visad.java3d import DirectManipulationRendererJ3D
import subs, jarray
from subs import makeColorMap
from java.awt import Color
from java.lang import Math
from visad.util import Util

class IndexChangeCell(CellImpl):
  def __init__(self, lineIdxRef, callback):
    self.lineIdxRef = lineIdxRef
    self.lockOut = 0
    self.callback = callback
    self.first = 0

  def lock(self):
    self.lockOut = 1

  def unlock(self):
    self.lockOut = 0

  def doAction(self):
    if self.first == 0:
      self.first = 1
      return

    if self.lockOut == 0:
      self.idx = int(self.lineIdxRef.getData().getValue())
      self.callback(self.idx)
    else:
      self.lockOut = 0


class DragablePointOnLine(CellImpl, DisplayListener):
  def __init__(self, pointRef, lineIdxRef, line, display, display_coord_sys, displayAltType, lon_idx=0, lat_idx=1,dspAltLev=0.0, altitudeMap=None):
    self.pointRef = pointRef
    self.lineIdxRef = lineIdxRef
    self.line     = line
    self.display  = display
    self.dsp_cs       = display_coord_sys
    self.lineSamples  = line.getSamples()
    self.first  = 0
    self.length = len(self.lineSamples[0])
    self.currentPoint = self.length/2
    self.nextPoint = -1
    self.remote_lineIdxRef = None
    self.lon_idx = lon_idx
    self.lat_idx = lat_idx
    self.dspAltLev = dspAltLev
    self.altitudeMap = altitudeMap


    point = pointRef.getData()
    if point != None:
      self.dragPoint = point
    else:
      self.altVal = (self.altitudeMap.inverseScaleValues([dspAltLev]))[0]
      self.dragPoint = RealTuple(RealTupleType(RealType.Latitude, RealType.Longitude, RealType.Altitude), [self.lineSamples[0][self.currentPoint], self.lineSamples[1][self.currentPoint],self.altVal])

      point = RealTuple(RealTupleType(RealType.Latitude, RealType.Longitude), [self.lineSamples[0][self.currentPoint], self.lineSamples[1][self.currentPoint]])

      pointRef.setData(point)

    self.dragPointRef = DataReferenceImpl('dragPoint')
    self.dragPointRef.setData(self.dragPoint)
    display.addReferences(DirectManipulationRendererJ3D(), self.dragPointRef, [ConstantMap(0.5,Display.PointSize)]+makeColorMap(Color(0.25,0.0,0.0)))
    
    self.display.addReference(pointRef, makeColorMap('green')+[ConstantMap(dspAltLev+0.02, displayAltType)]+[ConstantMap(6.0, Display.PointSize)])
    self.addReference(self.dragPointRef)
    self.lineSamples = self.dsp_cs.toReference([self.lineSamples[self.lat_idx], self.lineSamples[self.lon_idx], jarray.zeros(len(self.lineSamples[1]), 'f')])
    self.display.addDisplayListener(self)

  def setRemoteLineIdxRef(self, ref):
    self.remote_lineIdxRef = ref
    self.idxChangeCell = IndexChangeCell(self.remote_lineIdxRef, self.updateMarker)
    self.idxChangeCell.addReference(self.remote_lineIdxRef)

  def remove(self):
    self.display.removeDisplayListener(self)
    self.display.removeReference(self.pointRef)
    self.display.removeReference(self.dragPointRef)

  def updateMarker(self, idx):
    self.currentPoint = idx
    self.nextPoint = -1
    c_x = self.lineSamples[0][self.currentPoint]
    c_y = self.lineSamples[1][self.currentPoint]
    c_z = self.lineSamples[2][self.currentPoint]
    ll = self.dsp_cs.fromReference([[c_x], [c_y], [c_z]])
    point = RealTuple(RealTupleType(RealType.Latitude, RealType.Longitude), [ll[self.lat_idx][0], ll[self.lon_idx][0]])
    self.pointRef.setData(point)
    self.dragPoint = RealTuple(RealTupleType(RealType.Latitude, RealType.Longitude, RealType.Altitude), [ll[self.lat_idx][0], ll[self.lon_idx][0], self.altVal])
    self.dragPointRef.setData(self.dragPoint)

  def update(self, set, ds_cs=None, displayAltType=None):
    self.lineSamples = set.getSamples()
    self.length = len(self.lineSamples[0])
    self.currentPoint = self.length/2
    self.nextPoint = -1
    self.dsp_cs = ds_cs
    self.displayAltType = displayAltType

    point = RealTuple(RealTupleType(RealType.Latitude, RealType.Longitude), [self.lineSamples[1][self.currentPoint], self.lineSamples[0][self.currentPoint]])
    self.pointRef.setData(point)
    self.dragPoint = RealTuple(RealTupleType(RealType.Latitude, RealType.Longitude, RealType.Altitude), [self.lineSamples[1][self.currentPoint], self.lineSamples[0][self.currentPoint],self.altVal])
    self.lineSamples = self.dsp_cs.toReference([self.lineSamples[self.lat_idx], self.lineSamples[self.lon_idx], jarray.zeros(len(self.lineSamples[1]), 'f')])
   
  def initialize(self):
    self.length = len(self.lineSamples[0])
    self.currentPoint = self.length/2
    self.nextPoint = -1

    point = self.pointRef.getData()
    if point != None:
      self.dragPoint = point
    else:
      point = RealTuple(RealTupleType(RealType.Latitude, RealType.Longitude), [self.lineSamples[0][self.currentPoint], self.lineSamples[1][self.currentPoint]])
      self.pointRef.setData(point)
      self.dragPoint = RealTuple(RealTupleType(RealType.Latitude, RealType.Longitude, RealType.Altitude), [self.lineSamples[0][self.currentPoint], self.lineSamples[1][self.currentPoint],self.altVal])

    self.lineSamples = self.dsp_cs.toReference([self.lineSamples[1], self.lineSamples[0], jarray.zeros(len(self.lineSamples[1]), 'f')])
   

  def displayChanged(self, event):
    if (event.getId() == DisplayEvent.MOUSE_RELEASED):
      val = self.pointRef.getData().getValues()
      dragPoint = RealTuple(RealTupleType(RealType.Latitude, RealType.Longitude, RealType.Altitude), [val[0],val[1],self.altVal])
      self.dragPointRef.setData(dragPoint)
    
  def doAction(self):
    if self.first == 0:
      self.first = 1
      return
    ll_vals = self.dragPointRef.getData().getValues()
    xyz = self.dsp_cs.toReference([[ll_vals[self.lat_idx]], [ll_vals[self.lon_idx]], [ll_vals[2]]])
    x = xyz[0][0]
    y = xyz[1][0]
    z = xyz[2][0]
    c_x = self.lineSamples[0][self.currentPoint]
    c_y = self.lineSamples[1][self.currentPoint]
    c_z = self.lineSamples[2][self.currentPoint]

    del_x = self.lineSamples[0][self.currentPoint+1] - c_x
    del_y = self.lineSamples[1][self.currentPoint+1] - c_y
    del_z = self.lineSamples[2][self.currentPoint+1] - c_z
    dd = Math.sqrt(del_x*del_x+del_y*del_y+del_z*del_z)
    line_uvec0 = [del_x/dd, del_y/dd, del_z/dd]
    del_x = x - self.lineSamples[0][self.currentPoint]
    del_y = y - self.lineSamples[1][self.currentPoint]
    del_z = z - self.lineSamples[2][self.currentPoint]
    dd = Math.sqrt(del_x*del_x+del_y*del_y+del_z*del_z)
    if Util.isApproximatelyEqual(dd, 0.0) == 1: 
      return
    drg_line_uvec = [del_x/dd, del_y/dd, del_z/dd]
    dot = line_uvec0[0]*drg_line_uvec[0]+line_uvec0[1]*drg_line_uvec[1]+line_uvec0[2]*drg_line_uvec[2]

    if dot >= 0:
      self.nextPoint = self.currentPoint + 1
      self.dir = 1
    else:
      self.nextPoint = self.currentPoint - 1
      self.dir = -1

    while (1):
      pt_0 = self.nextPoint
      pt_1 = self.nextPoint+self.dir
      if (pt_1 < 0) or (pt_1 > self.length - 1):
        break
      del_x = self.lineSamples[0][pt_1] - self.lineSamples[0][pt_0]
      del_y = self.lineSamples[1][pt_1] - self.lineSamples[1][pt_0]
      del_z = self.lineSamples[2][pt_1] - self.lineSamples[2][pt_0]
      dd = Math.sqrt(del_x*del_x+del_y*del_y+del_z*del_z)
      line_uvec1 = [self.dir*del_x/dd, self.dir*del_y/dd, self.dir*del_z/dd]
      del_x = x - self.lineSamples[0][self.nextPoint]
      del_y = y - self.lineSamples[1][self.nextPoint]
      del_z = z - self.lineSamples[2][self.nextPoint]
      dd = Math.sqrt(del_x*del_x+del_y*del_y+del_z*del_z)
      drg_line_uvec = [del_x/dd, del_y/dd, del_z/dd]
      dot  = line_uvec1[0]*drg_line_uvec[0]+line_uvec1[1]*drg_line_uvec[1]+line_uvec1[2]*drg_line_uvec[2]

      if self.dir*dot >= 0:
        self.currentPoint = self.nextPoint
        self.nextPoint = self.currentPoint + self.dir
        c_x = self.lineSamples[0][self.currentPoint]
        c_y = self.lineSamples[1][self.currentPoint]
        c_z = self.lineSamples[2][self.currentPoint]
        ll = self.dsp_cs.fromReference([[c_x], [c_y], [c_z]])
        self.pointRef.setData(RealTuple(RealTupleType(RealType.Latitude, RealType.Longitude), [ll[self.lat_idx][0], ll[self.lon_idx][0]]))
        self.lineIdxRef.setData(Real(RealType.Generic, self.currentPoint))
      elif self.dir*dot < 0:
        break
