import sys

from javax.swing import JPopupMenu, JFrame, JMenuBar, JMenu, JMenuItem, JTextField, JFileChooser, JLabel, JPanel, JSplitPane, JButton
from javax.swing.border import LineBorder
from java.awt import GridLayout, BorderLayout, FlowLayout, Color, Dimension, Canvas, Toolkit 
from java.awt.event import InputEvent
from java.awt.datatransfer import DataFlavor
from java.lang import System, Runtime

from visad import *
from visad.bom import HydraImageRendererJ3D
import mapProjections
import subs
import graph
from subs import *
from visad.python.JPythonMethods import *
from ui.colorbar import ColorBarWidget, ColorBar

import MultiSpectralDataSource
import jarray
from hframe import HFrame
from subsetImage import SubsetImage, Subset
from histostretch import linkHistLocation, dragHistLocation

from util import Helper
from util import LambertEA, SatProjection, MapProjectionAdapter
from html_hydra import HtmlBrowserWindow

import hydra_properties
hydra_properties.setProperties()

false = 0
true  = 1

global adaptProj
import hydra_globals

runtime=Runtime.getRuntime()

class fireMaskDisplay:
  def __init__(self, displayData, subset=None):
    self.displayData=displayData
    self.currentProj = "Lambert Equal Area"
    self.currentLine = 0
    self.currentElem = 0
    self.isTestPath = 0
    self.currentSubset = None
    self.testRGBMap = None
    
    self.frame  = HFrame("MODIS Fire Mask Product",windowClosing=self.windowClose)
    self.framePane = self.frame.getContentPane()
    
    menuBar = JMenuBar()
    ##self.frame.setJMenuBar(menuBar)
    
    
    toolMenu = JMenu("Tools")
    toolItem = JMenuItem("Capture Display", actionPerformed=self.captureDisplay)
    toolItem.setEnabled(1)
    toolMenu.add(toolItem)
    menuBar.add(toolMenu)
    self.toolMenu = toolMenu
    
    self.observationsInstrument = self.displayData.imageRangeType

    if self.displayData.numObsElements < 512:
      self.currentSubset = None
    else:
      self.currentSubset = Subset(None,None,None,None,step=10)

    self.mcv = None
    if hydra_globals.mcViewer != None:
      #- overlay on existing MultiChannelViewer image display ---
      self.overlaySetup()
    else:
      print "cannot use fire mask viewer in standalone mode"
      ##self.standaloneSetup()

    #  mask toggle button 
    self.togText = ["Apply Mask to MODIS Image", "Turn Off Mask"]
    self.togIndex = 1
    self.lastTogIndex = 0
    buttonPanel = JPanel()
    buttonPanel.setLayout(GridLayout(1,2,0,0))
    self.mskToggleButton = JButton(self.togText[self.togIndex], actionPerformed=self.toggleCloudMask)
    if self.mcv == None:
      self.mskToggleButton.setEnabled(0)
      self.mskToggleButton.setText("Start Multi-Channel Viewer to Overlay")
    else:
      self.mskToggleButton.setEnabled(1)
      
    buttonPanel.add(self.mskToggleButton)
    self.framePane.add("South",buttonPanel)
  
  def overlaySetup(self):
    self.framePane.setLayout(BorderLayout())
    self.currentSubset = hydra_globals.mcViewer.currentSubset
    self.mcv = hydra_globals.mcViewer
    self.mcv.addUpdateListener(self)

    #- overlay color map, include transparancy:
    colors = [[],[],[],[]]
    fsmpls = jarray.zeros(10, 'f')
    fsmpls[0] = 0
    colors[0].append(0.5), colors[1].append(1.0), colors[2].append(0.0), colors[3].append(0.0)
    fsmpls[1] = 0.20
    colors[0].append(0.5), colors[1].append(1.0), colors[2].append(0.0), colors[3].append(0.0)
    fsmpls[2] = 0.21
    colors[0].append(0.7), colors[1].append(0.0), colors[2].append(0.7), colors[3].append(1.0)
    fsmpls[3] = 0.40
    colors[0].append(0.7), colors[1].append(0.0), colors[2].append(0.7), colors[3].append(1.0)
    fsmpls[4] = 0.41
    colors[0].append(1.0), colors[1].append(1.0), colors[2].append(0.0), colors[3].append(1.0)
    fsmpls[5] = 0.60
    colors[0].append(1.0), colors[1].append(1.0), colors[2].append(0.0), colors[3].append(1.0)
    fsmpls[6] = 0.61
    colors[0].append(1.0), colors[1].append(0.7), colors[2].append(0.2), colors[3].append(1.0)
    fsmpls[7] = 0.80
    colors[0].append(1.0), colors[1].append(0.7), colors[2].append(0.2), colors[3].append(1.0)
    fsmpls[8] = 0.81
    colors[0].append(1.0), colors[1].append(0.0), colors[2].append(0.0), colors[3].append(1.0)
    fsmpls[9] = 1.00
    colors[0].append(1.0), colors[1].append(0.0), colors[2].append(0.0), colors[3].append(1.0)
    clr_set = Gridded1DSet(RealType.Generic, [fsmpls], len(fsmpls))
    clr_fnc = FlatField(FunctionType(RealType.Generic, RealTupleType([makeType("cR"), makeType("cG"), makeType("cB"), makeType("cA")])), clr_set)
    clr_fnc.setSamples(colors)

    self.rgbMap = ScalarMap(self.observationsInstrument, Display.RGBA)
    self.mcv.d.addMap(self.rgbMap)
    cc = self.rgbMap.getControl()
    cc.setFunction(clr_fnc)
    self.img = self.makeMaskField()
    self.cldMskRef = subs.addData('firemask', self.img, self.mcv.d, constantMaps=[ConstantMap(-0.2,self.mcv.displayAltType)], renderer=HydraImageRendererJ3D())
    self.ovlyClr_fnc = clr_fnc
    self.currentSubset = self.mcv.currentSubset
    self.maskColors=colors

    from javax.swing import SwingConstants
    colorBar = ColorBar(self.rgbMap,false)
    colorBar.setPreferredSize(Dimension(20,400))
    colorBar.setLayout(GridLayout(1,2,0,0))
    cbLabelPanel=JPanel(GridLayout(5,1,0,0))
    cbLabelPanel.setPreferredSize(Dimension(20,400))
    ##self.labelsToAdd = ["Clear","Probably Clear","Uncertain","Cloudy"]
    self.labelsToAdd = ["Non-fire", "Not processed","Low","Nominal","High"]
    for i in xrange(len(self.labelsToAdd)):
      cbLabelPanel.add(JLabel(self.labelsToAdd[i],SwingConstants.LEFT))

    colorBar.add(cbLabelPanel)
    self.framePane.add("Center", colorBar)
    self.colorBar = colorBar
    self.cbLabelPanel = cbLabelPanel
    self.toolMenu.setVisible(0)
    
    self.frame.pack()
    self.frame.show()

  def makeMaskField(self):
    mcvImg = self.mcv.refToImage.getData()
    msk = self.displayData.getImageSegment(0,self.currentSubset)
    new_mtype = FunctionType(mcvImg.getType().getDomain(), msk.getType().getRange())
    new_field = FlatField(new_mtype, mcvImg.getDomainSet())
    new_field.setSamples(Helper.unpackFloats(msk,true), false)
    return new_field
 

  def rebuildCloudMaskDisplay(self):
     if self.displayData.byteIndex: # for test overlay
       img = self.displayData.getImageSegment(0, self.currentSubset)

       if not hasattr(self,"maps"):
         self.mapToUse = self.rgbMap
       else:
         self.mapToUse = self.maps[2]
         
       self.curDisp.disableAction()
                
       testColors=[[],[],[],[]]
       updateColors=[[],[],[],[]]
       fsmpls = jarray.zeros(self.displayData.cloudMaskByte0SampleLen+6, 'f')
       #fsmpls[0] = 0
       testColors[0].append(1.0), testColors[1].append(1.0), testColors[2].append(0.0), testColors[3].append(0.0)
       #fsmpls[1] = 0.33
       testColors[0].append(1.0), testColors[1].append(1.0), testColors[2].append(0.0), testColors[3].append(0.0)
       #fsmpls[2] = 0.34
       testColors[0].append(1.0), testColors[1].append(0.0), testColors[2].append(1.0), testColors[3].append(1.0)
       #fsmpls[3] = 0.66
       testColors[0].append(1.0), testColors[1].append(0.0), testColors[2].append(1.0), testColors[3].append(1.0)
       #fsmpls[4] = 0.67
       testColors[0].append(0.0), testColors[1].append(0.0), testColors[2].append(0.0), testColors[3].append(1.0)
       #fsmpls[5] = 1.00
       testColors[0].append(0.0), testColors[1].append(0.0), testColors[2].append(0.0), testColors[3].append(1.0)
       # based on the length of cloud mask byte 0 field that the test field is overlaying
       if self.displayData.cloudMaskByte0SampleLen==8:
          fsmpls=[0, 0.10, 0.11, 0.25,0.26, 0.40,0.41, 0.55,0.56,0.70,0.71,0.85,0.86, 1]
          self.mapToUse.setRange(0,6)
       elif self.displayData.cloudMaskByte0SampleLen==4:
          fsmpls=[0, 0.20, 0.21, 0.40,0.41, 0.60,0.61, 0.80,0.81, 1]     
          self.mapToUse.setRange(0,4)
                                  
       clr_set = Gridded1DSet(RealType.Generic, [fsmpls], len(fsmpls))
       clr_fnc = FlatField(FunctionType(RealType.Generic, RealTupleType([makeType("cR"), makeType("cG"), makeType("cB"), makeType("cA")])), clr_set)
       for i in xrange(len(updateColors)):
         updateColors[i]=self.maskColors[i]+testColors[i]
         
       clr_fnc.setSamples(updateColors)
                     
       if self.testRGBMap != None:
          # remove the map and the reference     
          self.curDisp.removeMap(self.testRGBMap)
          self.curDisp.removeReference(self.testMaskRef)
          # make sure test toggle text is set correctly
          self.testTogIndex = 1
          self.lastTestTogIndex = 0
          self.testMaskToggleButton.setText(self.testTogText[self.testTogIndex])
       
       self.testMaskToggleButton.setEnabled(1)                    
       self.testRGBMap = ScalarMap(self.displayData.testRangeType,Display.RGBA)    
       self.curDisp.addMap(self.testRGBMap)
       testcc = self.mapToUse.getControl()
       #test2=testcc.getFunction()
       testcc.setFunction(clr_fnc)
       #cc = self.testRGBMap.getControl()
       #cc.setFunction(clr_fnc)
                
       self.testMaskRef = subs.addData('cmTest', img, self.curDisp,renderer=HydraImageRendererJ3D(),zlayer=-0.1)      
       self.testRGBMap.setRange(0,6)  #- to make sure the test mask values appear as the right color. 
       self.overlayTestColor_fnc = clr_fnc
       
       self.colorBar.remove(self.cbLabelPanel)
       
       cbLabelPanel=JPanel(GridLayout(7-(4 - self.displayData.cloudMaskByte0SampleLen/2),1,0,0))
       cbLabelPanel.setPreferredSize(Dimension(20,400))
       for i in xrange(len(self.labelsToAdd)):
          cbLabelPanel.add(JLabel(self.labelsToAdd[i])) 
       
       # for some reason, getRealTypes doesn't like decimal points, so have to do this
       tempSplitName=self.displayData.paramName.replace("<point>",".").split("_")
       # print out the name for the specific tests applied
       splitName = ""
       for i in xrange(len(tempSplitName)):
         splitName = splitName + tempSplitName[i] + "<P>"
          
       cbLabelPanel.add(JLabel("<html>" + splitName + "Test Not Applied"))
       cbLabelPanel.add(JLabel("<html>" + splitName + "Cloud Found"))
       cbLabelPanel.add(JLabel("<html>" + splitName + "Cloud Not Found"))
       # add labels to the right of the color bar
       self.colorBar.add(cbLabelPanel)
       self.cbLabelPanel=cbLabelPanel
       
       if self.mcv != None:
         #self.framePane.remove(self.colorBar)
         #self.framePane.add("Center",self.colorBar)
         self.frame.pack()
         #self.frame.setSize(350,500)
       else:
         #self.splitPane.add(self.colorBar)          
         self.splitPane.setDividerLocation(250)
       
       self.curDisp.enableAction()
       
       #hydra_globals.dtuple_cnt += 1
     else: # replace with displays from Cloud Mask Byte
        # clear display before creating a new one
       if self.mcv != None:
         self.curDisp.removeMap(self.rgbMap)
         self.curDisp.removeReference(self.cldMskRef)
       else:  
         self.cloudMaskDisp.removeAllReferences()
         self.cloudMaskDisp.clearMaps()
        # reset self.testRGBMap each time loading in different bits of byte 0
         self.testRGBMap = None
         # get rid of the right pane (colorbar) if it exists
         testForColorbar=self.splitPane.getRightComponent()
         if testForColorbar != None:
           self.splitPane.remove(testForColorbar)
       
       # disable the test toggle button if reloading
       if hasattr(self,"testMaskToggleButton"):
         self.testMaskToggleButton.setEnabled(0)
        
       img = self.displayData.getImageSegment(0, self.currentSubset)
               
       if self.mcv != None:
         self.rgbMap = ScalarMap(self.observationsInstrument, Display.RGBA)
         self.curDisp.addMap(self.rgbMap)
         self.mapToUse = self.rgbMap
       else:  
         corners = mapProjections.getValidCorners(self.displayData.gs, [[-180,180], [-90, 90]])
        
         minLon = min(corners[0])
         minLat = min(corners[1])
         maxLon = max(corners[0])
         maxLat = max(corners[1])
             
         displayLatType   = DisplayRealType("ProjectionLat_%d" % hydra_globals.dtuple_cnt, true, minLat, maxLat, 0.0, CommonUnit.degree)
         displayLonType   = DisplayRealType("ProjectionLon_%d" % hydra_globals.dtuple_cnt, true, minLon, maxLon, 0.0, CommonUnit.degree)
         displayAltType   = DisplayRealType("ProjectionAlt_%d" % hydra_globals.dtuple_cnt, true, -1.0, 1.0, 0.0, None)
        
         adaptProj = MapProjectionAdapter(LambertEA(corners))
        
         self.ds_cs = adaptProj
        
         displayTupleType = DisplayTupleType([displayLonType, displayLatType, displayAltType], adaptProj)
        
         self.maps = subs.makeMaps(self.obsLongitudeInstrument,'x', self.obsLatitudeInstrument, 'y', self.observationsInstrument, 'rgba')
             
         #- replace first two with user-defined display types
         self.maps[0] = ScalarMap(self.obsLongitudeInstrument, displayLonType)
         self.maps[1] = ScalarMap(self.obsLatitudeInstrument, displayLatType)
         #- for rubber band zoomer
         self.maps.append(ScalarMap(RealType.XAxis, Display.XAxis))
         self.maps.append(ScalarMap(RealType.YAxis, Display.YAxis))
         self.maps[3].setRange(-1, 1)
         self.maps[4].setRange(-1, 1)      
         self.curDisp.addMaps(self.maps)
        # get the dimensions of the domain
         dimInstrumentDomain = img.getDomainSet()
         self.dimInstrument = dimInstrumentDomain.getLengths()
         self.maps[0].setRange(minLon, maxLon)
         self.maps[1].setRange(minLat, maxLat)  
         self.mapToUse = self.maps[2]
                  
       self.curDisp.disableAction()
                           
       self.cldMskRef=subs.addData('a',img,self.curDisp,renderer=HydraImageRendererJ3D(), zlayer = -0.2)
       if self.mcv != None:
         transparentFlag = 0.0
       else:  
         transparentFlag = 1.0      
         subs.addData('b',self.map,self.curDisp,constantMaps=makeColorMap(hydra_properties.foreground_color))
             
       colors = [[],[],[],[]]
       if self.displayData.paramName == "Unobstructed_FOV_Quality_Flag":
          fsmpls = jarray.zeros(8, 'f')
          fsmpls = [0.00,0.25,0.26,0.50,0.51,0.75,0.76,1.00]
          colors[0].append(0.5), colors[1].append(1.0), colors[2].append(0.0), colors[3].append(transparentFlag)
          colors[0].append(0.5), colors[1].append(1.0), colors[2].append(0.0), colors[3].append(transparentFlag)
          
          colors[0].append(0.0), colors[1].append(1.0), colors[2].append(1.0), colors[3].append(1.0)
          colors[0].append(0.0), colors[1].append(1.0), colors[2].append(1.0), colors[3].append(1.0)
          
          colors[0].append(1.0), colors[1].append(0.0), colors[2].append(0.0), colors[3].append(1.0)
          colors[0].append(1.0), colors[1].append(0.0), colors[2].append(0.0), colors[3].append(1.0)
          
          colors[0].append(1.0), colors[1].append(1.0), colors[2].append(1.0), colors[3].append(1.0)
          colors[0].append(1.0), colors[1].append(1.0), colors[2].append(1.0), colors[3].append(1.0)
          cbLabelPanel=JPanel(GridLayout(4,1,0,0))
          cbLabelPanel.setPreferredSize(Dimension(25,400))
          self.labelsToAdd = ["Clear","Probably Clear","Uncertain","Cloudy"]
          self.mapToUse.setRange(0,3)
          
       elif self.displayData.paramName == "Cloud_Mask_Determined":
          fsmpls = jarray.zeros(4,'f')
          fsmpls = [0.00, 0.5, 0.51, 1.00]
          colors[0].append(0.5), colors[1].append(1.0), colors[2].append(0.0), colors[3].append(1.0)
          colors[0].append(0.5), colors[1].append(1.0), colors[2].append(0.0), colors[3].append(1.0)
          
          colors[0].append(0.0), colors[1].append(1.0), colors[2].append(1.0), colors[3].append(1.0)
          colors[0].append(0.0), colors[1].append(1.0), colors[2].append(1.0), colors[3].append(1.0)
          self.labelsToAdd = ["Not Determined","Determined"]
          self.mapToUse.setRange(0,1)
          
       elif self.displayData.paramName == "Sunglint":
          fsmpls = jarray.zeros(4,'f')
          fsmpls = [0.00, 0.5, 0.51, 1.00]
          colors[0].append(0.5), colors[1].append(1.0), colors[2].append(0.0), colors[3].append(1.0)
          colors[0].append(0.5), colors[1].append(1.0), colors[2].append(0.0), colors[3].append(1.0)
          
          colors[0].append(0.0), colors[1].append(1.0), colors[2].append(1.0), colors[3].append(transparentFlag)
          colors[0].append(0.0), colors[1].append(1.0), colors[2].append(1.0), colors[3].append(transparentFlag)
          self.labelsToAdd = ["Sunglint","No Sunglint"]
          self.mapToUse.setRange(0,1)
          
       elif self.displayData.paramName == "Snow/Ice_Background":
          fsmpls = jarray.zeros(4,'f')
          fsmpls = [0.00, 0.5, 0.51, 1.00]
          colors[0].append(0.5), colors[1].append(1.0), colors[2].append(0.0), colors[3].append(1.0)
          colors[0].append(0.5), colors[1].append(1.0), colors[2].append(0.0), colors[3].append(1.0)
          
          colors[0].append(0.0), colors[1].append(1.0), colors[2].append(1.0), colors[3].append(transparentFlag)
          colors[0].append(0.0), colors[1].append(1.0), colors[2].append(1.0), colors[3].append(transparentFlag)
          self.labelsToAdd=["Snow/Ice","No Snow/Ice"]
          self.mapToUse.setRange(0,1)
          
       elif self.displayData.paramName == "Day_or_Night":
          fsmpls = jarray.zeros(4,'f')
          fsmpls = [0.00, 0.5, 0.51, 1.00]
          colors[0].append(0.5), colors[1].append(1.0), colors[2].append(0.0), colors[3].append(1.0)
          colors[0].append(0.5), colors[1].append(1.0), colors[2].append(0.0), colors[3].append(1.0)
          
          colors[0].append(0.0), colors[1].append(1.0), colors[2].append(1.0), colors[3].append(1.0)
          colors[0].append(0.0), colors[1].append(1.0), colors[2].append(1.0), colors[3].append(1.0)
          self.labelsToAdd = ["Night","Day"]
          self.mapToUse.setRange(0,1)
          
       elif self.displayData.paramName == "Land_or_Water":
          fsmpls = jarray.zeros(8, 'f')
          fsmpls = [0.00,0.25,0.26,0.50,0.51,0.75,0.76,1.00]
          colors[0].append(0.5), colors[1].append(1.0), colors[2].append(0.0), colors[3].append(transparentFlag)
          colors[0].append(0.5), colors[1].append(1.0), colors[2].append(0.0), colors[3].append(transparentFlag)
          
          colors[0].append(0.0), colors[1].append(1.0), colors[2].append(1.0), colors[3].append(1.0)
          colors[0].append(0.0), colors[1].append(1.0), colors[2].append(1.0), colors[3].append(1.0)
          
          colors[0].append(1.0), colors[1].append(0.0), colors[2].append(0.0), colors[3].append(1.0)
          colors[0].append(1.0), colors[1].append(0.0), colors[2].append(0.0), colors[3].append(1.0)
          
          colors[0].append(1.0), colors[1].append(1.0), colors[2].append(1.0), colors[3].append(1.0)
          colors[0].append(1.0), colors[1].append(1.0), colors[2].append(1.0), colors[3].append(1.0)
          self.labelsToAdd=["Land","Desert","Coastal","Water"]
          self.mapToUse.setRange(0,3)
                  
       cbLabelPanel=JPanel(GridLayout(len(fsmpls)/2,1,0,0))
       cbLabelPanel.setPreferredSize(Dimension(25,400))
       for i in xrange(len(self.labelsToAdd)):
          cbLabelPanel.add(JLabel(self.labelsToAdd[i]))      
          
       clr_set = Gridded1DSet(RealType.Generic, [fsmpls], len(fsmpls))
                                                                                                                                                  
       clr_fnc = FlatField(FunctionType(RealType.Generic, RealTupleType([makeType("cR"), makeType("cG"), makeType("cB"), makeType("cA")])), clr_set)
       clr_fnc.setSamples(colors)
       cc = self.mapToUse.getControl()
       cc.setFunction(clr_fnc)
       self.maskColors=colors
       self.displayData.cloudMaskByte0SampleLen=len(fsmpls)
       self.cbLabelPanel=cbLabelPanel

       self.ovlyClr_fnc = clr_fnc
               
       # add labels to the right of the color bar
       colorBar = ColorBar(self.mapToUse,false)
       colorBar.setPreferredSize(Dimension(25,400))
       colorBar.setLayout(GridLayout(1,2,0,0))
       colorBar.add(cbLabelPanel)
        
       if self.mcv != None:
         self.framePane.remove(self.colorBar)
         self.framePane.add("Center",colorBar)
         self.frame.pack()
         # make sure toggle button is set correctly
         self.togIndex = 1
         self.lastTogIndex = 0
         self.mskToggleButton.setText(self.togText[self.togIndex])
         #self.frame.setSize(350,500)
       else:
         self.splitPane.add(colorBar)
        
       hydra_globals.dtuple_cnt += 1
       self.colorBar=colorBar
       self.cbLabelPanel=cbLabelPanel
       self.img=img
              
       #self.displayAltType = displayAltType
         
     self.curDisp.enableAction()
     
  def captureDisplay(self,event):
    Helper.saveDisplay(self.cloudMaskDisp, JFrame().getContentPane())

  def windowClose(self,event):
    if self.mcv != None:
      self.mcv.d.removeReference(self.cldMskRef)
      self.mcv.d.removeMap(self.rgbMap)
      if hasattr(self,"testMaskRef"):
        self.mcv.d.removeReference(self.testMaskRef)
        self.mcv.d.removeMap(self.testRGBMap)
      self.mcv.removeUpdateListener(self)

    self.frame.dispose()
    runtime.gc()
          
  def toggleCloudMask(self, event):
    if self.mcv != None:
      if (self.togIndex == 0):
        self.mcv.d.toggle(self.cldMskRef, 1)
      if (self.togIndex == 1):
        self.mcv.d.toggle(self.cldMskRef, 0)
      self.mskToggleButton.setText(self.togText[self.lastTogIndex])
      tmp = self.togIndex
      self.togIndex = self.lastTogIndex
      self.lastTogIndex = tmp

  def updateDisplay(self, subset=None):
    if subset != None:
      self.currentSubset = subset
      self.img = self.makeMaskField()

    self.mcv.d.removeReference(self.cldMskRef)
    self.mcv.d.removeMap(self.rgbMap)
    self.rgbMap = ScalarMap(self.observationsInstrument, Display.RGBA)
    self.mcv.d.addMap(self.rgbMap)
    self.cc = self.rgbMap.getControl()
    self.cc.setFunction(self.ovlyClr_fnc)
    self.cldMskRef = subs.addData('cld_msk', self.img, self.mcv.d, constantMaps=[ConstantMap(-0.2, self.mcv.displayAltType)], renderer=HydraImageRendererJ3D())
