from java.awt import Color
from visad import ControlListener, DisplayListener, DataReferenceImpl, Gridded2DSet, RealTupleType, UnionSet
from visad import RealType, TextType, TupleType, Real, Text, Tuple, DisplayEvent, ScalarMap, Display
from visad.java3d import MouseBehaviorJ3D
from java.lang import Math
import subs
import jarray
import java

from hydra_properties import true, false
from hydra_globals import color_scale_dict


class lonlatgrid(ControlListener, DisplayListener):
  def __init__(self, display, dsp_cs, colorScale):
    self.display = display
    self.dsp_cs  = dsp_cs
    self.mer_lbl_refs = []
    self.par_lbl_refs = []
    self.rot   = jarray.zeros(3, 'd')
    self.scale = jarray.zeros(1, 'd')
    self.trans = jarray.zeros(3, 'd')
    self.last_scale = jarray.zeros(1, 'd')
    self.last_trans = jarray.zeros(3, 'd')
    self.scale_ratio = 1.20
    self.trans_dist = 0.30
    self.enabled = false
    self.gridRef = None
    self.colorScale = colorScale
 
    dr = self.display.getDisplayRenderer()
    self.mouseBehavior = dr.getMouseBehavior()
    self.canvas = dr.getCanvas()

    pCntrl = self.display.getProjectionControl()
    self.pCntrl = pCntrl

    self.lnltTxtType = TextType.getTextType("lonlat_labels")
    sclrMap = ScalarMap(self.lnltTxtType, Display.Text)
    self.display.addMap(sclrMap)
    self.txtCntrl = sclrMap.getControl()
    self.txtCntrl.setAutoSize(1)
    self.txtMap = sclrMap

  def resetTextMap(self):
    self.display.removeMap(self.txtMap)
    sclrMap = ScalarMap(self.lnltTxtType, Display.Text)
    self.display.addMap(sclrMap)
    self.txtCntrl = sclrMap.getControl()
    self.txtCntrl.setAutoSize(1)
    self.txtMap = sclrMap

  def enable(self, dsp_cs=None):
    self.enabled = true
    self.mouseReleased = false
    if dsp_cs != None:
      self.dsp_cs = dsp_cs
                                                                                                                              
    matrix = self.pCntrl.getMatrix()
    MouseBehaviorJ3D.unmake_matrix(self.rot, self.scale, self.trans, matrix)
    self.last_trans[0] = self.trans[0]
    self.last_trans[1] = self.trans[1]
    self.last_trans[2] = self.trans[2]
    self.last_scale[0] = self.scale[0]
                                                                                                                              
    self.gridRef = DataReferenceImpl("lon lat grid")
    color = (color_scale_dict[self.colorScale.current])['lon_lat_grid_color']
    self.display.addReference(self.gridRef, subs.makeColorMap(color))
                                                                                                                              
    mer_color = (color_scale_dict[self.colorScale.current])['lon_label_color']
    par_color = (color_scale_dict[self.colorScale.current])['lat_label_color']
    for i in xrange(3):
      lbl_ref = DataReferenceImpl("lon lat label")
      self.display.addReference(lbl_ref, subs.makeColorMap(mer_color))
      self.mer_lbl_refs.append(lbl_ref)
                                                                                                                              
      lbl_ref = DataReferenceImpl("lon lat label")
      self.display.addReference(lbl_ref, subs.makeColorMap(par_color))
      self.par_lbl_refs.append(lbl_ref)
                                                                                                                              
    self.display.addDisplayListener(self)
    self.pCntrl.addControlListener(self)

    self.update()

  def __del__(self):
    self.disable()

  def disable(self):
    self.enabled = false
    self.display.removeReference(self.gridRef)
    for i in xrange(len(self.mer_lbl_refs)):
      self.display.removeReference(self.mer_lbl_refs[i])
    for i in xrange(len(self.par_lbl_refs)):
      self.display.removeReference(self.par_lbl_refs[i])
    self.display.removeDisplayListener(self)
    self.pCntrl.removeControlListener(self)

    self.mer_lbl_refs = []
    self.par_lbl_refs = []
    self.gridRef = None

  def controlChanged(self, event):
    if (event.getControl() != self.pCntrl):
      return
    matrix = self.pCntrl.getMatrix()
    MouseBehaviorJ3D.unmake_matrix(self.rot, self.scale, self.trans, matrix)
    if (self.mouseReleased == true):
      ratio = self.scale[0]/self.last_scale[0]
      dist = Math.sqrt((self.trans[0]-self.last_trans[0])**2 + (self.trans[1]-self.last_trans[1])**2)
      if (ratio > self.scale_ratio or ratio < 1/self.scale_ratio):
        self.last_trans[0] = self.trans[0]
        self.last_trans[1] = self.trans[1]
        self.last_trans[2] = self.trans[2]
        self.last_scale[0] = self.scale[0]
        self.update()
        self.mouseReleased = false
      elif (abs(dist) > self.trans_dist):
        self.update()
        self.last_trans[0] = self.trans[0]
        self.last_trans[1] = self.trans[1]
        self.last_trans[2] = self.trans[2]
        self.last_scale[0] = self.scale[0]
        self.update()
        self.mouseReleased = false


  def displayChanged(self, event):
    id = event.getId()
    if id == DisplayEvent.MOUSE_RELEASED:
      self.mouseReleased = true
      ratio = self.scale[0]/self.last_scale[0]
      dist = Math.sqrt((self.trans[0]-self.last_trans[0])**2 + (self.trans[1]-self.last_trans[1])**2)
      if (ratio > self.scale_ratio or ratio < 1/self.scale_ratio):
        self.last_trans[0] = self.trans[0]
        self.last_trans[1] = self.trans[1]
        self.last_trans[2] = self.trans[2]
        self.last_scale[0] = self.scale[0]
        self.update()
        self.mouseReleased = false
      elif (abs(dist) > self.trans_dist):
        self.last_trans[0] = self.trans[0]
        self.last_trans[1] = self.trans[1]
        self.last_trans[2] = self.trans[2]
        self.last_scale[0] = self.scale[0]
        self.update()
        self.mouseReleased = false


  def update(self, dsp_cs=None):
    if self.enabled == false:
      return

    self.size = self.canvas.getSize()
    height = self.size.height
    width  = self.size.width
    scrn_coords = [[0,0], [width-1,0], [0, height-1], [width-1, height-1]]
    data_coords = []
    self.lola_coords = [[],[]]
    if dsp_cs != None:
      self.dsp_cs = dsp_cs

    for i in xrange(len(scrn_coords)):
      vray = self.mouseBehavior.findRay(scrn_coords[i][0], scrn_coords[i][1])
      x = vray.position[0]
      y = vray.position[1]
      data_coords.append([x,y])
      lola = self.dsp_cs.fromReference([[x],[y],[0]])
      self.lola_coords[0].append(lola[0][0])
      self.lola_coords[1].append(lola[1][0])

    #-- pole in screen ?
    self.pole_in_screen = false
    height = self.size.height
    width = self.size.width
    cntr_w = width/2
    cntr_h = height/2
    vray = self.mouseBehavior.findRay(cntr_w, cntr_h)
    x = vray.position[0]
    y = vray.position[1]
    lola = self.dsp_cs.fromReference([[x],[y],[0]])
    cntr_lon = lola[0][0]
    cntr_lat = lola[1][0]
    d1 = cntr_lat - self.lola_coords[1][0]
    d2 = self.lola_coords[1][3] - cntr_lat
    if d1*d2 <= 0:
      self.pole_in_screen = true
    d1 = cntr_lat - self.lola_coords[1][1]
    d2 = self.lola_coords[1][2] - cntr_lat
    if d1*d2 <= 0:
      self.pole_in_screen = true

    uset = self.makeGrid()
    self.display.disableAction()
    self.gridRef.setData(uset)
    self.makeMeridianLabels()
    self.makeParallelLabels()
    self.display.enableAction()

  def makeGrid(self, intrvl=None):
    intervals   = [40.0,35.0,30.0,25.0,20.0,15.0,10.0,8.0,5.0,4.0,3.0,2.0,1.5,1.0,0.5,0.4,0.2,0.1,0.05]
    n_pts       = 20
    n_meridians = 5
    n_parallels = 5
    lon_base    = 0
    lat_base    = 0
    self.g_sets   = []
    self.mer_sets = []
    self.par_sets = []
    
    d_lon    = []
    d_lat    = []
    min_lons = []
    max_lons = []
    min_lats = []
    max_lats = []

    # use corners to estimate lon/lat range
    for i in xrange(3):
      for j in xrange(i+1,4):
        mnLon = min(self.lola_coords[0][i], self.lola_coords[0][j])
        mxLon = max(self.lola_coords[0][i], self.lola_coords[0][j])
        min_lons.append(mnLon)
        max_lons.append(mxLon)
        d_lon.append(mxLon-mnLon)
        mnLat = min(self.lola_coords[1][i], self.lola_coords[1][j])
        mxLat = max(self.lola_coords[1][i], self.lola_coords[1][j])
        min_lats.append(mnLat)
        max_lats.append(mxLat)
        d_lat.append(mxLat-mnLat)

    for i in xrange(len(d_lon)):
      if (d_lon[i] > 180):
        d_lon[i] -= 360
      d_lon[i] = abs(d_lon[i])

    #- use the widest lon/lat ranges for start/stop
    max_lon_diff = max(d_lon)
    ii = d_lon.index(max_lon_diff)
    min_lon = min_lons[ii]
    max_lon = max_lons[ii]

    max_lat_diff = max(d_lat)
    ii = d_lat.index(max_lat_diff)
    min_lat = min_lats[ii]
    max_lat = max_lats[ii]

    #-- pole in screen ?
    if (self.pole_in_screen == true):
      min_lat = min_lat
      max_lat = 89.8
      if min_lat < 0:
        max_lat = min_lat
        min_lat = -89.8

    lon_diff = max_lon - min_lon
    if (lon_diff > 180):
      lon_diff -= 360
    mer_inc = (abs(lon_diff))/(n_meridians-1)
    diff_min = 360
    i_i = 0
    for i in xrange(len(intervals)):
      diff = abs(mer_inc - intervals[i])
      if diff <= diff_min:
        i_i = i
        diff_min = diff
    mer_inc = intervals[i_i]

    if (lon_diff < 0):
      mer_inc *= -1.0

    par_inc = (max_lat - min_lat)/(n_parallels-1)
    diff_min = 360
    i_i = 0
    for i in xrange(len(intervals)):
      diff = abs(par_inc - intervals[i])
      if diff <= diff_min:
        i_i = i
        diff_min = diff
    par_inc = intervals[i_i]


    min_lon = int(((min_lon-mer_inc)-lon_base)/mer_inc)*mer_inc
    min_lat = int(((min_lat-par_inc)-lat_base)/par_inc)*par_inc
    lon_diff = ((max_lon+mer_inc) - min_lon)
    if lon_diff > 180:
      lon_diff -= 360
    nm = int(lon_diff/mer_inc)
    np = int(((max_lat+par_inc) - min_lat)/par_inc)
    max_lon = min_lon + (nm+1)*mer_inc
    max_lat = min_lat + (np+1)*par_inc
    n_meridians = nm+1
    n_parallels = np+1

    if self.pole_in_screen:
      min_lon = -170
      max_lon = 170
      n_meridians = 18
      mer_inc = 20
      n_mer_pts = 50
    else:
      n_mer_pts = n_pts
    
    n_pts = n_mer_pts
    #--- make the lines  ---------------------------
    lat_inc = ((max_lat+par_inc) - (min_lat-par_inc))/(n_pts-1)
    lon = min_lon
    for i in xrange(n_meridians):
      lon_array = jarray.zeros(n_pts, 'f')
      lat_array = jarray.zeros(n_pts, 'f')
      lat = min_lat - par_inc
      for j in xrange(n_pts):
        if (lon < -180):
          lon_array[j] = lon + 360
        elif (lon > 180):
          lon_array[j] = lon - 360
        else:
          lon_array[j] = lon
        lat_array[j] = lat

        lat += lat_inc
      lon += mer_inc
      self.mer_sets.append(Gridded2DSet(RealTupleType.SpatialEarth2DTuple, [lon_array, lat_array], n_pts))

    lon_inc = ((max_lon+mer_inc) - (min_lon-mer_inc))/(n_pts-1)
    lat = min_lat
    for i in xrange(n_parallels):
      lon_array = jarray.zeros(n_pts, 'f')
      lat_array = jarray.zeros(n_pts, 'f')
      lon = min_lon - mer_inc
      for j in xrange(n_pts):
        if (lon < -180):
          lon_array[j] = lon + 360
        elif (lon > 180):
          lon_array[j] = lon - 360
        else:
          lon_array[j] = lon
        lat_array[j] = lat

        lon += lon_inc
      lat += par_inc
      self.par_sets.append(Gridded2DSet(RealTupleType.SpatialEarth2DTuple, [lon_array, lat_array], n_pts))
    return UnionSet(self.mer_sets + self.par_sets)

  def makeMeridianLabels(self):
    hght = self.size.height
    wdth = self.size.width

    n_lbls = 0
    for j in xrange(len(self.mer_sets)):
      vals     = self.mer_sets[j].getSamples()
      vals_3d  = [vals[0], vals[1], jarray.zeros(len(vals[0]), 'f')]
      vad_vals = self.dsp_cs.toReference(vals_3d)
      lbl_xy = None
      k = -1
      for i in xrange(len(vad_vals[0])):
        x = vad_vals[0][i]
        y = vad_vals[1][i]
        z = vad_vals[2][i]
        scrn_xy = self.mouseBehavior.getScreenCoords([x, y, z])
        if ((scrn_xy[0] >= 0 and scrn_xy[0] <= wdth-1) and (scrn_xy[1] >= 0 and scrn_xy[1] <= hght-1)):
          lbl_xy = scrn_xy
          k = i
          break

      if k != -1 and n_lbls < 3:
        label = Tuple(TupleType([RealType.Longitude, RealType.Latitude, self.lnltTxtType]), [Real(RealType.Longitude, vals[0][k]), Real(RealType.Latitude, vals[1][k]), Text(self.lnltTxtType, '   %-4.1f' % vals[0][k])])
        self.mer_lbl_refs[n_lbls].setData(label)
        n_lbls += 1

  def makeParallelLabels(self):
    hght = self.size.height
    wdth = self.size.width

    n_lbls = 0
    for j in xrange(len(self.par_sets)):
      vals     = self.par_sets[j].getSamples()
      vals_3d  = [vals[0], vals[1], jarray.zeros(len(vals[0]), 'f')]
      vad_vals = self.dsp_cs.toReference(vals_3d)
      lbl_xy = None
      k = -1
      for i in xrange(len(vad_vals[0])):
        x = vad_vals[0][i]
        y = vad_vals[1][i]
        z = vad_vals[2][i]
        scrn_xy = self.mouseBehavior.getScreenCoords([x, y, z])
        if ((scrn_xy[0] >= 0 and scrn_xy[0] <= wdth-1) and (scrn_xy[1] >= 0 and scrn_xy[1] <= hght-1)):
          lbl_xy = scrn_xy
          k = i
          break

      if k != -1 and n_lbls < 3:
        label = Tuple(TupleType([RealType.Longitude, RealType.Latitude, self.lnltTxtType]), [Real(RealType.Longitude, vals[0][k]), Real(RealType.Latitude, vals[1][k]), Text(self.lnltTxtType, '   %-4.1f' % vals[1][k])])
        self.par_lbl_refs[n_lbls].setData(label)
        n_lbls += 1
