import sys, jarray

sys.path.append('multispectral/Lib')
sys.path.append('multispectral/Lib/adde')
sys.path.append('multispectral/Lib/file')

from visad import DisplayRealType, DisplayTupleType 
from visad import CommonUnit, Data
from visad import *
from visad.data.hdfeos import PolarStereographic
from util import LambertEA, MapProjectionAdapter, SatProjection, PolarStereo

import MultiSpectralDataSource
import subs
from subs import *
import java
from visad.bom import ImageRendererJ3D
from visad.bom import HydraImageRendererJ3D

rt = java.lang.Runtime.getRuntime()


class MultiSpectralControl:
  def __init__(self, dataSource):
    self.dataSource = dataSource
    self.animMap = None
    self.rgbMap = None

    obsElementInstrument, obsLineInstrument, obsChannelIndexInstrument, observationChannelsInstrument, observationsInstrument = dataSource.getRealTypes()

    image = dataSource.getImageSegment(0)

    from java.util import HashMap
    cacheMap = HashMap()
    cacheLen = 170
    domainSet = Integer1DSet(RealType.Generic, cacheLen)
    f_type = FunctionType(RealType.Generic, image.getType())
    field = FieldImpl(f_type, domainSet)
    for i in xrange(cacheLen):
      spectralIndex = 230+(2*i)
      cacheIndex = i
      spectralValue = dataSource.observationChannels[spectralIndex]
      cacheMap.put(spectralValue, cacheIndex)
      image = dataSource.getSameImageSegmentCopy(spectralIndex)
      field.setSample(i, image, 0)
      print i, cacheLen

    self.imageCacheRef = DataReferenceImpl('imageCacheRef')
    self.imageCacheRef.setData(field)

    import spectrumDisplay
    from spectrumDisplay import SpectrumDisplay

    spectrumDisplay = SpectrumDisplay(dataSource)
    spectralValueRef = spectrumDisplay.leftSelectRef
    self.cacheMap = cacheMap
    self.spectralValueRef = spectralValueRef
    self.dataReference = self.imageCacheRef
    self.rgbRealType = observationsInstrument

    rt.gc()

    self.positionRef = DataReferenceImpl('positionRef')

    class PositionUpdateCell(CellImpl):
      def __init__(self, dataRef, dataSource):
        self.dataRef = dataRef
        self.dataSource = dataSource

      def doAction(self):
        latlon = self.dataRef.getData()
        if latlon == None:
          return
        rcomps = latlon.getRealComponents()
        lat = rcomps[0].getValue()
        lon = rcomps[1].getValue()
        xy = self.dataSource.cs.fromReference([[lat,],[lon,]])
        numLine = xy[0][0]
        numElem = xy[1][0]
                                                                                                                               
        imageDomain = self.dataSource.imageField.getDomainSet()
        lens = imageDomain.getLengths()
        dim = len(lens)
        idx = imageDomain.valueToIndex([[numLine], [numElem]])[0]
        grid = []
                                                                                                                               
        k = idx
        for j in xrange(dim-1):
          grid.append( k % lens[j])
          k = k / lens[j]
        grid.append(k)
                                                                                                                               
        try:
          currentLine = int(numLine)
          currentElem = int(numElem)
        except: #- off image
          return
        numLine = int(grid[0])
        numElem = int(grid[1])
                                                                                                                               
        if numLine >= 0 and numLine < lens[0] and numElem >= 0 and numElem < lens[1]:
          self.dataSource.getObservations(numLine,numElem)


    pos_update_cell = PositionUpdateCell(self.positionRef, self.dataSource)
    pos_update_cell.addReference(self.positionRef)

  def setColorMap(self, rgbMap):
    self.rgbMap = rgbMap

  def setAnimationMap(self, animMap):
    self.animMap = animMap

    class LeftCell(CellImpl):
      def __init__(self, spectralValueRef, animMap, cacheMap):
        self.spectralValueRef = spectralValueRef
        self.acontrol = animMap.getControl()
        self.cacheMap = cacheMap
        self.animMap  = animMap

      def doAction(self):
        val = self.spectralValueRef.getData().getValue()
        cacheIndex = self.cacheMap.get(val)
        self.acontrol = self.animMap.getControl()
        if self.acontrol == None:
          return
        if cacheIndex != None:
          self.acontrol.setCurrent(cacheIndex)

    lcell = LeftCell(self.spectralValueRef, animMap, self.cacheMap)
    lcell.addReference(self.spectralValueRef)

  def setAnimation(self, anim):
    self.anim = anim
                                                                                                                               
    class LeftCell(CellImpl):
      def __init__(self, spectralValueRef, anim, cacheMap):
        self.spectralValueRef = spectralValueRef
        self.cacheMap = cacheMap
        self.anim = anim
                                                                                                                               
      def doAction(self):
        val = self.spectralValueRef.getData().getValue()
        cacheIndex = self.cacheMap.get(val)
        if cacheIndex != None:
          self.anim.setCurrent(cacheIndex)
                                                                                                                               
    lcell = LeftCell(self.spectralValueRef, anim, self.cacheMap)
    lcell.addReference(self.spectralValueRef)


  def setSelectRangeMap(self, animMap):
    self.animMap = animMap
                                                                                                                               
    class LeftCell(CellImpl):
      def __init__(self, spectralValueRef, animMap, cacheMap):
        self.spectralValueRef = spectralValueRef
        self.acontrol = animMap.getControl()
        self.cacheMap = cacheMap
        self.animMap  = animMap
                                                                                                                               
      def doAction(self):
        val = self.spectralValueRef.getData().getValue()
        cacheIndex = self.cacheMap.get(val)
        self.acontrol = self.animMap.getControl()
        if self.acontrol == None:
          return
        if cacheIndex != None:
          self.acontrol.setValue(cacheIndex)
                                                                                                                               
    lcell = LeftCell(self.spectralValueRef, animMap, self.cacheMap)
    lcell.addReference(self.spectralValueRef)
