import MultiSpectralDataSource
from MultiSpectralDataSource import MultiSpectralData

class MOD06_CloudOpticalThickness(MultiSpectralData):
  def __init__(self, filename, reader, readerTable,master=None):
    MultiSpectralData.__init__(self, filename, reader, readerTable, master)
                                                                                                                                  
    if master == None:
      reader_name  = reader.reader.__class__
      reader_name  = reader_name.__name__
                                                                                                                                  
      if ((filename.find("MYD") != -1) or (filename.find("a1.") != -1)):
        self.modis_platform = "Aqua"
      else:
        self.modis_platform = "Terra"
                                                                                                                                  
    else:
      self.geo_reader     = master.geo_reader
      self.geoDimOrder    = master.geoDimOrder
      self.modis_platform = master.modis_platform

  def getScaleOffset(self):
    self.paramScale   = []
    self.paramOffset  = []
                                                                                                                                  
    self.fieldToRetrieve=self.imageRangeTypeDict.get(self.paramName).toString()
    paramScale, paramOffset   = self.reader.getObsScaleOffset(self.fieldToRetrieve, "scale_factor", "add_offset")
    if paramScale == None or paramOffset == None:
      self.paramScale.append(None)
      self.paramOffset.append(None)
    else:
      scales  = jarray.zeros(len(paramScale), 'f')
      offsets = jarray.zeros(len(paramOffset), 'f')
      for i in xrange(len(scales)):
         scales[i]  = paramScale[i]
         offsets[i] = paramOffset[i]
                                                                                                                                  
      self.paramScale.append(scales)
      self.paramOffset.append(offsets)

  def extractValues(self,dataToExtract):
      valuesToExtract=jarray.zeros(len(dataToExtract),'f')
      self.getScaleOffset()
      for i in xrange(len(dataToExtract)):
        if dataToExtract[i] < -9998:
          valuesToExtract[i]=Float.NaN
        else:
          valuesToExtract[i]=self.paramScale[0][0]*(dataToExtract[i]-self.paramOffset[0][0])
                                                                                                                                  
      return valuesToExtract

  def makeRangeTypes(self):
    self.rangeTypes = []
    paramNames=["Cloud Optical Thickness"]
    self.imageRangeTypeDict={}
    for i in xrange(self.numObsDataSets):
        self.rangeTypes.append(RealType.getRealType(self.observationName[i]))
        self.imageRangeTypeDict[paramNames[i]]=RealType.getRealType(self.observationName[i])
    self.spectrumRangeTypeDict    = None
                                                                                                                                  
  def makeSpectrumRangeType(self, paramName):
    self.spectrumRangeType = None
                                                                                                                                  
  def getNativeParam(self):
    native_param="Cloud Optical Thickness"
    return  native_param
                                                                                                                                  
  def setDefaultParam(self):
    return self.native_param
                                                                                                                                  
  def getObsValidRange(self): #- valid is probably a misnomer here; mostly for setRange
    if self.paramName   == "Cloud Top Pressure":
      return [0, 1000]
    elif self.paramName == "Cloud Top Temperature":
      return [180, 320]
    elif self.paramName == "Cloud Effective Radius":
      return [0, 1000]
    elif self.paramName == "Cloud Phase":
      return None

  def getImageRange(self, values):
    self.imageRangeUnit = None
    return values
                                                                                                                                  
  def changeParameter(self, paramName):
    self.paramName = paramName
                                                                                                                                  
  def clone(self):
    return MOD06_CloudOpticalThickness(self.filename,self.reader,self.readerTable,master=self)
