import jarray
import struct
from util import Helper
from visad import *
from visad.util import DataUtility
from java.io import FileInputStream
from java.util import Properties
from visad.python.JPythonMethods import *
from java.lang import Float, Math, System, Object
from java.net import URL
from visad.data.units import *

from subsetImage import Subset

false = 0
true  = 1

from MultiSpectralDataSource import MultiSpectralData


class ModisFireMaskData(MultiSpectralData):
  def __init__(self, filename, reader, readerTable,master=None):
    MultiSpectralData.__init__(self, filename, reader, readerTable, master)
        
    #-look for MODIS geo file in the same directory as the MOD35 Product
    if master == None:
      reader_name  = reader.reader.__class__
      reader_name  = reader_name.__name__
      
      if ((filename.find("MYD") != -1) or (filename.find("a1.") != -1)):
        self.modis_platform = "Aqua"
      else:
        self.modis_platform = "Terra"
      self.geoOffset = 2
    
    else:
      self.geo_reader     = master.geo_reader
      self.geoDimOrder    = master.geoDimOrder
      self.modis_platform = master.modis_platform
      self.geoOffset      = master.geoOffset

  def makeTrackSegments(self):
    if self.numObsLines < 2600:
      self.numberSegments = 1
      self.trackSegments = [1, self.numObsLines]
      return
    self.numberSegments = 1
    self.trackSegments = [401, self.numObsLines-100]

  
  # overwrite
  def extractValues(self,dataToExtract,cloudMaskBits,qaDataToExtract,qaBits):   
    # get bit at specified position
    def getBitAt(byteVal,index):    
      # negative integers in Java represented in 2's complement
      # hdf integers written as 2's complement and big-endian
      #b=byteVal & 0x0f
      #mask = 1 << index
      #print b, mask, b & mask
      #if b & mask == mask:
      #   return 1
      #else:
      #   return 0
      return (byteVal >> index) & 1
    
    outMask=jarray.zeros(len(dataToExtract),'f')
       
    for i in xrange(len(dataToExtract)):
      bitVal=[]      
      for j in xrange(len(cloudMaskBits)):
        bitVal.append(getBitAt(dataToExtract[i],int(cloudMaskBits[j])))
             
      # This is true only for Unobstructed FOV Quality Flag and Land or Water Path
      # no quality assurance for these as well
      if len(bitVal) > 1:
        test1=bitVal[1] and bitVal[0]
        test2=bitVal[1] and (not bitVal[0])
        test3=not bitVal[1] and bitVal[0]
        test4=not bitVal[1] and (not bitVal[0])
        if test1:
          outMask[i]=0
        elif test2:
          outMask[i]=1
        elif test3:
          outMask[i]=2
        elif test4:
          outMask[i]=3
      else:
        if qaBits != None:
          qaVal = getBitAt(qaDataToExtract[i],int(qaBits))
          testNotApplied = not(qaVal)
          testAppliedCloudFound = qaVal and not(bitVal[0])
          testAppliedCloudNotFound = qaVal and bitVal[0]
          if testNotApplied:
            outMask[i] = 4
          elif testAppliedCloudFound:
            outMask[i] = 5
          elif testAppliedCloudNotFound:
            outMask[i] = 6
          # adjust depending on what cloud mask byte 0 field selected
          # maximum number of cloud mask byte 0 fields is 4
          outMask[i]=outMask[i] - (4 - self.cloudMaskByte0SampleLen/2)  
        else:
          outMask[i]=bitVal[0]
            
    return outMask
    
  def makeRangeTypes(self):
    self.rangeTypes = []
    self.rangeTypes.append(RealType.getRealType("firemask"))    
    self.imageRangeTypeDict = {"firemask" : RealType.getRealType("firemask")}
    self.spectrumRangeTypeDict    = None

  def makeImageRangeType(self, paramName):
    self.imageRangeType = self.imageRangeTypeDict["firemask"] 

  def makeSpectrumRangeType(self, paramName):
    self.spectrumRangeType = None
  
  def getImageRange(self, values):
    new_values = jarray.zeros(len(values), 'f')
    for i in xrange(len(values)):
      vv = values[i]
      new_vv = Float.NaN
      if vv >= 3 and vv <= 5:
        new_vv = 0
      elif vv >= 0 and vv <= 2:
        new_vv = 1
      elif vv == 7:
        new_vv = 2
      elif vv == 8:
        new_vv = 3
      elif vv == 9:
        new_vv = 4
      new_values[i] = float(new_vv)

    self.imageRangeUnit = None
    return new_values
    
  def getObsValidRange(self): #- valid is probably a misnomer here; mostly for setRange
     return [0, 4]
    
  def getNativeParam(self):
    native_param = "firemask"
    return native_param

  def setDefaultParam(self):
    return self.native_param
  
  def getImageSegment(self, segmentIndex=0, subset=None):
    self.currentSegmentIndex = segmentIndex   
    segStartLine = int(self.trackSegments[self.currentSegmentIndex*2]-1)  #convert one to zero based
    segEndLine   = int(self.trackSegments[self.currentSegmentIndex*2+1]-1)
    
    if subset != None: #-- subset logic
      self.subset = subset      
      if subset.end_x == None or subset.end_y == None:
        self.startElem = 0
        self.nElements = int((self.numObsElements)/subset.step)
        self.startLine = segStartLine
        self.nLines    = int((int(segEndLine-self.startLine+1))/subset.step)
      else:                 
        t_line  = subset.start_x
        t_elem  = subset.start_y
        n_lines = int((subset.end_x - subset.start_x)/subset.step) + 1
        n_elems = int((subset.end_y - subset.start_y)/subset.step) + 1
        if (self.invert == true):
          t_elem  = subset.start_x
          t_line  = subset.start_y
          n_elems = int((subset.end_x - subset.start_x)/subset.step) + 1
          n_lines = int((subset.end_y - subset.start_y)/subset.step) + 1
        
        self.startElem = t_elem
        self.startLine = segStartLine + t_line      
        self.nElements = n_elems
        self.nLines    = n_lines
      
      self.step      = self.subset.step
           
      if float(self.ratio)/float(self.step) <= 1:
        self.geo_step = int(1.0/(float(self.ratio)/float(self.step)))
        nGeoLines     = self.nLines
        nGeoElements  = self.nElements
      else:
        self.geo_step = 1
        nGeoLines     = int(float(self.nLines)/float(self.ratio)/float(self.step))
        nGeoElements  = int(float(self.nElements)/float(self.ratio)/float(self.step))
        
    else:
      self.subset    = None
      self.startElem = 0
      self.nElements = self.numObsElements
      self.startLine = segStartLine
      self.nLines    = int(segEndLine-self.startLine+1)
      
      self.step      = 1  
      
      if float(self.ratio)/float(self.step) <= 1:
        self.geo_step = int(1.0/(float(self.ratio)/float(self.step)))
        nGeoLines     = self.nLines
        nGeoElements  = self.nElements
      else:
        self.geo_step = 1
        nGeoLines     = int(float(self.nLines)/float(self.ratio)/float(self.step))
        nGeoElements  = int(float(self.nElements)/float(self.ratio)/float(self.step))
   
    start   = [self.startElem+self.geoOffset,self.startLine+self.geoOffset]
    count   = [self.nElements,self.nLines]
    stride  = [self.step, self.step]
    nstart  = [None, None]
    ncount  = [None, None]
    nstride = [None, None]
    
    for i in xrange(len(start)):
      nstart[self.obsDimOrder[i]]  = start[i]
      ncount[self.obsDimOrder[i]]  = count[i]
      nstride[self.obsDimOrder[i]] = stride[i]
      
    if self.step > 1: #- this hack will go away when Java-Netcdf implements stride reading
      self.z = self.reader.getValues(self.observationName[0], nstart, ncount, nstride)
    else:
      self.z = self.reader.getValues(self.observationName[0], nstart, ncount, None)
          
    ##self.z=self.extractValues(self.maskValues,cloudMaskBits,self.qaValues,qaBits)
            
    if self.invert == true:
      self.ylength  = self.nLines
      self.xlength  = self.nElements
      self.gylength = nGeoLines
      self.gxlength = nGeoElements
    else:
      self.xlength  = self.nLines
      self.ylength  = self.nElements
      self.gxlength = nGeoLines
      self.gylength = nGeoElements

    if self.ratio == 1:
      self.gxlength = self.xlength
      self.gylength = self.ylength
     
    self.imageDomType = RealTupleType(self.spatialTupleArray[0], self.spatialTupleArray[1])
    self.imageDomain  = makeDomain(self.imageDomType,
          0,((self.xlength)/(self.xlength/self.gxlength))-1,self.xlength,
          0,((self.ylength)/(self.ylength/self.gylength))-1,self.ylength)
      
      
    self.imageFunctionType = FunctionType(self.imageDomType, self.imageRangeType)
    self.imageField        = FlatField(self.imageFunctionType, self.imageDomain)  
    self.z                 = self.getImageRange(self.z)
        
    self.imageField.setSamples([self.z], false)
       
    return self.imageField
      
  def clone(self):
    return ModisFireMaskData(self.filename,self.reader,self.readerTable,master=self)
