import jarray
from util import Helper
from visad import *
from visad.util import DataUtility
from java.io import FileInputStream
from java.util import Properties
from visad.python.JPythonMethods import *
from java.lang import Double, Float, Math, System, Object
from java.net import URL
from visad.data.units import *
from visad.data.mcidas import AreaForm, AreaAdapter
from edu.wisc.ssec.mcidas import AreaFileFactory
from subsetImage import Subset
import hydra_globals
from hydra_globals import hydra_path


false = 0
true  = 1
    

def load(datasetname, date_str, time_str):
  if datasetname.startswith('file:'): # otherwise fall through to server request
    file = AreaFileFactory.getFileFromURLString(datasetname)

    areaFile = AreaFileFactory.getAreaFileInstance(file)
    areaDir = areaFile.getAreaDirectory()
    lines = areaDir.getLines()
    elems = areaDir.getElements()
    id = areaDir.getSensorID()
    date = areaDir.getNominalTime()

    date_str = date.toString()
    time_str = ''
    
    datasetname = AreaFileFactory.makeLocalSubsetString(file,0,lines,1,0,elems,1,1,'RAW')
    
    if id == 51 or id == 52:
      return MsgData(datasetname, date_str, time_str)
    if id == 74:
      return GOES10_Imager(datasetname, date_str, time_str)
    if id == 75:
      return GOES10_Sounder(datasetname, date_str, time_str)
    if id == 78:
      return GOESE_Data(datasetname, date_str, time_str)
    if id == 70:
      return GOESW_Data(datasetname, date_str, time_str)
    

  rq = ADDErequest(datasetname)
  group = rq.getGroup()

  if group == 'msg':
    return MsgData(datasetname, date_str, time_str)
  if group == 'EUM_AD':
    return MsgData(datasetname, date_str, time_str)
  if group == 'BEDKA':
    return MsgData(datasetname, date_str, time_str)
  if group == 'EAST':
    return GOESE_Data(datasetname, date_str, time_str)
  if group == 'AGOES12':
    return GOESE_Data(datasetname, date_str, time_str)
  if group == 'WEST':
    return GOESW_Data(datasetname, date_str, time_str)
  if group == 'FLYOVER':
    desc = rq.getDesc()
    if desc == 'ALL':
      return AVHRR3_Data(datasetname, date_str, time_str)
  if group == 'GILMORE':
    desc = rq.getDesc()
    return AVHRR3_Data(datasetname, date_str, time_str)


class MultiSpectralData:
  def __init__(self, filename, date_str, time_str, master=None):
    if master == None:
      self.filename            = filename
      self.addeRq              = ADDErequest(filename)
      self.subset              = None
      self.instrumentName      = None
      
      self.observationName     = None
      self.obsElementName      = "ImageElement"
      self.obsLineName         = "ImageLine"
      self.obsLatitudeName     = "Latitude"
      self.obsLongitudeName    = "Longitude"
      self.obsTimeOfDayName    = None
      self.obsFOVangleName     = None

      self.makeRealTypes()
      self.makeSpectrumDomainType()

      self.numObsLines, self.numObsElements = self.addeRq.getSize()     
      
        
      #- get channels arrays
      self.band_numbers                = None
      self.observationChannels         = None
      self.observationChannels         = self.getObservationChannels()
      self.numChannelsPerDataSet       = [self.numObsChannels]
      self.observationChannelIntervals = None

    
      #- init geo range
      self.obsLonMinMax        = None
      self.obsLatMinMax        = None
      self.currentSpectrum     = None
      self.currentImage        = None
      self.currentImageCS      = None
      self.observationField    = None
      self.observationDomain   = None
    
      #--------------------------------------------------------------------------
      self.obsIsRadiance = true         #- false indicates brightness temperature
      self.native_param = "radiance"
      self.paramName    = "brightnessTemp"
      self.makeRangeTypes()
      self.makeSpectrumRangeType(self.paramName)
      self.makeImageRangeType(self.paramName)
      self.area_form = AreaForm()
      self.numberSegments = 1
      self.invert = true
      self.reqUnits = self.reqUnitDict[self.paramName]
      self.res_ratio  = 1
      self.nav_offset = 1
      self.date_str = date_str
      self.time_str = time_str
      self.unnavigated = false
    else:
      self.copy(master)
  #-----------------------------------------------------------------------------
  
  def copy(self, master):
    self.filename            = master.filename

    self.instrumentName      = master.instrumentName
    self.observationName     = master.observationName
    self.obsElementName      = master.obsElementName
    self.obsLineName         = master.obsLineName
    self.obsLatitudeName     = master.obsLatitudeName
    self.obsLongitudeName    = master.obsLongitudeName
    self.obsTimeOfDayName    = master.obsTimeOfDayName
    self.obsFOVangleName     = master.obsFOVangleName

    self.obsElementType      = master.obsElementType
    self.obsLineType         = master.obsLineType
    self.obsLongitudeType    = master.obsLongitudeType
    self.obsLatitudeType     = master.obsLatitudeType
    self.obsChannelsType     = master.obsChannelsType

    self.numObsElements      = master.numObsElements
    self.numObsLines         = master.numObsLines
    self.numObsChannels      = master.numObsChannels

    self.observationChannels = master.observationChannels
    self.observationChannelIntervals = master.observationChannelIntervals

    self.obsLonMinMax        = master.obsLonMinMax
    self.obsLatMinMax        = master.obsLatMinMax
    self.currentSpectrum     = master.currentSpectrum
    self.currentImage        = master.currentImage
    self.currentImageCS      = master.currentImageCS
    self.observationField    = None
    self.obsIsRadiance       = master.obsIsRadiance
    self.native_param        = master.native_param
    self.paramName           = master.paramName
    self.imageRangeType        = master.imageRangeType
    self.spectrumRangeType     = master.spectrumRangeType
    self.observationDomain     = master.observationDomain
    self.area_form             = master.area_form
    self.addeRq                = master.addeRq
    self.numberSegments        = master.numberSegments
    self.invert                = master.invert 
    self.broadBand             = master.broadBand
    self.numObsDataSets        = master.numObsDataSets
    self.numChannelsPerDataSet = master.numChannelsPerDataSet
    self.band_numbers          = master.band_numbers
    self.reqUnits              = master.reqUnits
    self.ratio                 = master.ratio
    self.res_ratio             = master.res_ratio
    self.nav_offset            = master.nav_offset
    self.date_str              = master.date_str
    self.time_str              = master.time_str
    self.unnavigated           = master.unnavigated
    
  def clone(self):
    return MultiSpectralData(self.filename, self.date_str, self.time_str, master=self)   
  
  #--- VisAD RealTypes
  def makeRealTypes(self):
    name = self.obsElementName
    self.obsElementType       = getRealType(name)

    name = self.obsLineName
    self.obsLineType          = getRealType(name)

    self.obsLongitudeType     = RealType.Longitude
    self.obsLatitudeType      = RealType.Latitude
    
      
  def makeSpectrumDomainType(self):
    centimeter = ScaledUnit(0.01, CommonUnit.meter, "cm")
    inv_cm = centimeter.pow(-1)
    inv_cm = ScaledUnit(1, inv_cm, "cm^-1")
    self.obsChannelsType = getRealType("wavenumber", inv_cm)

  def makeRangeTypes(self):
    self.rangeTypes = []
    self.rangeTypes.append(RealType.getRealType("radiance"))
    self.rangeTypes.append(RealType.getRealType("brightnessTemp"))    
    self.spectrumRangeTypeDict = {"radiance" : RealType.getRealType("radiance"), "brightnessTemp" : RealType.getRealType("brightnessTemp")}
    self.imageRangeTypeDict    = self.spectrumRangeTypeDict


  def makeSpectrumRangeType(self, paramName):
    self.spectrumRangeType = self.spectrumRangeTypeDict[paramName]
    
  def makeImageRangeType(self, paramName):
    self.imageRangeType = self.imageRangeTypeDict[paramName]
    
    
  def getObservationChannels(self):
    self.fileIndexes = None
    if self.observationChannels == None:
      if self.obsChannelsName != None:
        self.observationChannels = []
        for i in xrange(self.numObsDataSets):
          self.observationChannels += list(self.reader.getValues(self.obsChannelsName[i]))
      else:
        self.observationChannels = [i for i in xrange(self.numObsChannels)]
        
    return self.observationChannels
    

  def getObservationChannelIntervals(self): 
    if self.observationChannelIntervals == None:
      self.observationChannelIntervals = self.reader.getValues(self.obsChannelIntervalsName)
    return self.observationChannelIntervals
    

  def getObservations(self, line, element, copy=false):   
    vals = None
    #-- make the VisAD data object
    if self.observationField == None or copy == true:
      self.observationDomain = Gridded1DSet(self.obsChannelsType, [self.observationChannels], self.numObsChannels)

      if self.broadBand == false:
        self.observationField = FlatField(FunctionType(self.obsChannelsType, self.spectrumRangeType), self.observationDomain)
        if vals != None: self.observationField.setSamples([vals])
      else:
        if (isinstance(self.spectrumRangeType, RealTupleType)):
          fields = []
          for i in xrange(self.spectrumRangeType.getDimension()):
            range_type = RealTupleType([self.obsChannelsType]+[self.spectrumRangeType.getComponent(i)])
            ff = FlatField(FunctionType(RealType.Generic, range_type), Integer1DSet(self.numObsChannels))
            if vals != None:
              ff.setSamples([self.observationChannels, vals[i]])
            fields.append(ff)
          self.observationField = Tuple(fields)
        else:
          range_type = RealTupleType(self.obsChannelsType, self.spectrumRangeType)
          self.observationField = FlatField(FunctionType(RealType.Generic, range_type ), Integer1DSet(self.numObsChannels))
          if vals != None:
            self.observationField.setSamples([self.observationChannels, vals])
    else:
      if self.broadBand == false:
        self.observationField.setSamples([vals])
      else:
        if (isinstance(self.spectrumRangeType, RealTupleType)):
          for i in xrange(self.spectrumRangeType.getDimension()):
            if vals != None:
              self.observationField.getComponent(i).setSamples([self.observationChannels, vals[i]])
        else:
          if vals != None:
            self.observationField.setSamples([self.observationChannels, vals])

    return self.observationField

    
  def getImageSegment(self, channelIndex, segmentIndex=0, subset=None):
    if channelIndex < 0:
      return None

    dataset_index = self.channelIndexToDataSetIndex(channelIndex)
    channelIndex  = self.channelIndexToFileIndex(dataset_index, channelIndex)

    req = None
    if subset != None:
      if subset.end_x == None or subset.end_y == None:

        lines = self.numObsLines/self.ratio[channelIndex]
        lines /= (subset.step)
        elems = self.numObsElements/self.ratio[channelIndex]
        elems /= (subset.step)

        #-crappy hack for MSG ADDE server problem:
        if self.instrumentName == "MSG":
          lines -= lines % 4
          elems -= elems % 4
        
        req = self.addeRq.newRequest(lines, elems/self.res_ratio,
                                     lmag=-1*subset.step*self.ratio[channelIndex],
                                     emag=-1*subset.step*self.ratio[channelIndex]*self.res_ratio,
                                     lin=0, ele=0,
                                     band=self.band_numbers[channelIndex],
                                     unit=self.reqUnits[channelIndex])
        self.xlength   = elems
        self.ylength   = lines
        self.startLine = 0
        self.startElem = 0
        self.step      = subset.step
      else:
        t_elem  = subset.start_x
        t_line  = subset.start_y

        #-crappy hack for MSG ADDE server problem:
        if self.instrumentName == "MSG":
          t_elem -= t_elem % 4
          t_line -= t_line % 4

        n_elems = int((subset.end_x - subset.start_x)/subset.step) + 1
        n_lines = int((subset.end_y - subset.start_y)/subset.step) + 1

        #-crappy hack for MSG ADDE server problem:
        if self.instrumentName == "MSG":
          n_elems -= n_elems % 4
          n_lines -= n_lines % 4


        if n_lines < 0:
          t_line  = subset.end_y
          n_lines = -1*n_lines

        req = self.addeRq.newRequest(lines=n_lines, elems=n_elems/self.res_ratio,
                                     lmag=-1*subset.step*self.ratio[channelIndex],
                                     emag=-1*subset.step*self.ratio[channelIndex]*self.res_ratio,
                                     lin=t_line*self.ratio[channelIndex], ele=t_elem*self.ratio[channelIndex],
                                     band=self.band_numbers[channelIndex],
                                     unit=self.reqUnits[channelIndex])

        self.xlength = n_elems
        self.ylength = n_lines
        self.startLine = t_line
        self.startElem = t_elem
        self.step = subset.step
    else:
      req          = self.filename
      self.xlength = self.numObsElements
      self.ylength = self.numObsLines
      self.startLine = 0
      self.startElem = 0

    self.gxlength = self.xlength
    self.gylength = self.ylength
    ff = self.area_form.open(req)

    if self.unnavigated == false:
      self.imageDomType = ff.getType().getDomain()
      self.imageDomain  = ff.getDomainSet()
    else:
      rttype = ff.getType().getDomain()
      rcomps = rttype.getRealComponents()
      self.imageDomType = RealTupleType(rcomps)
      dom = ff.getDomainSet()
      lens = dom.getLengths()
      l0 = dom.getLinear1DComponent(0)
      l1 = dom.getLinear1DComponent(1)
      self.imageDomain = Linear2DSet(self.imageDomType, l0.getFirst(), l0.getLast(), lens[0], l1.getFirst(), l1.getLast(), lens[1])

    self.cs = ff.getType().getDomain().getCoordinateSystem()
    f_type = FunctionType(self.imageDomType, self.imageRangeType)
    new_ff = FlatField(f_type, self.imageDomain)
    vals = (Helper.unpackFloats(ff, false))[0]
    self.missingObs = self.initMissing()
    vals = self.missingObs.removeMissing(vals)
    vals = self.getImageRange(vals, self.paramName, dataset_index, channelIndex)
    new_ff.setSamples([vals], false)
    self.imageField = new_ff

    self.subsetDomain = makeDomain(self.imageDomType,
          0, (self.xlength-1)*self.step, self.xlength/self.res_ratio,
          (self.ylength-1)*self.step, 0, self.ylength)

    indexes                = jarray.zeros(2, 'i')
    minmax                 = Helper.minmax(vals, indexes)
    minloc = self.cs.toReference(self.imageDomain.indexToValue([indexes[0]]))
    maxloc = self.cs.toReference(self.imageDomain.indexToValue([indexes[1]]))
    self.obsMin = [minloc[0][0], minloc[1][0], minmax[0]]
    self.obsMax = [maxloc[0][0], maxloc[1][0], minmax[1]]

    self.req_ff   = ff
    self.rng_vals = vals

    return self.imageField


  def setUnnavigatedImage(self, flag):
    self.unnavigated = flag

  def getUnnavigatedImage(self):
    rttype = self.req_ff.getType().getDomain()
    rcomps = rttype.getRealComponents()
    self.imageDomType = RealTupleType(rcomps)
    ftype = FunctionType(self.imageDomType, self.imageRangeType)
    dom = ff.getDomainSet()
    lens = dom.getLengths()
    l0 = dom.getLinear1DComponent(0)
    l1 = dom.getLinear1DComponent(1)
    self.imageDomain = Linear2DSet(self.imageDomType, l0.getFirst(), l0.getLast(), lens[0], l1.getFirst(), l1.getLast(), lens[1])
    self.imageField = FlatField(f_type, self.imageDomain)
    self.imageField.setSamples([self.rng_vals], false)
    return self.imageField

  def getNavigatedImage(self):
    self.imageDomType = self.req_ff.getType().getDomain()
    f_type = FunctionType(self.imageDomType, self.imageRangeType)
    self.imageDomain = self.req_ff.getDomainSet()
    self.imageField = FlatField(f_type, self.imageDomain)
    self.imageField.setSamples([self.rng_vals], false)
    return self.imageField


  def getSameImageSegment(self, channelIndex):
    if channelIndex >= 0:
      dataset_index = self.channelIndexToDataSetIndex(channelIndex)
      channelIndex  = self.channelIndexToFileIndex(dataset_index, channelIndex)

      req = self.addeRq.newRequest(lines=self.ylength, elems=self.xlength/self.res_ratio,
                                   lmag=-1*self.step*self.ratio[channelIndex], emag=-1*self.step*self.ratio[channelIndex]*self.res_ratio,
                                   lin=self.startLine*self.ratio[channelIndex], ele=self.startElem*self.ratio[channelIndex],
                                   band=self.band_numbers[channelIndex],
                                   unit=self.reqUnits[channelIndex])


      ff = self.area_form.open(req)
      vals = (Helper.unpackFloats(ff, false))[0]
      vals = self.missingObs.removeMissing(vals)
      vals = self.getImageRange(vals, self.paramName, dataset_index, channelIndex)
      self.imageField.setSamples([vals], false)
                                                                                                                                   
      indexes                = jarray.zeros(2, 'i')
      minmax                 = Helper.minmax(vals, indexes)
      minloc = self.cs.toReference(self.imageDomain.indexToValue([indexes[0]]))
      maxloc = self.cs.toReference(self.imageDomain.indexToValue([indexes[1]]))
      self.obsMin = [minloc[0][0], minloc[1][0], minmax[0]]
      self.obsMax = [maxloc[0][0], maxloc[1][0], minmax[1]]


  def getSameImageSegmentCopy(self, channelIndex):
    if channelIndex >= 0:
      dataset_index = self.channelIndexToDataSetIndex(channelIndex)
      channelIndex  = self.channelIndexToFileIndex(dataset_index, channelIndex)

      req = self.addeRq.newRequest(lines=self.ylength, elems=self.xlength/self.res_ratio,
                                   lmag=-1*self.step*self.ratio[channelIndex], emag=-1*self.step*self.ratio[channelIndex]*self.res_ratio,
                                   lin=self.startLine*self.ratio[channelIndex], ele=self.startElem*self.ratio[channelIndex],
                                   band=self.band_numbers[channelIndex],
                                   unit=self.reqUnits[channelIndex])

      ff = self.area_form.open(req)
      vals = (Helper.unpackFloats(ff, false))[0]
      vals = self.missingObs.removeMissing(vals)
      vals = self.getImageRange(vals, self.paramName, dataset_index, channelIndex)

      ##f_type = FunctionType(ff.getType().getDomain(), self.imageRangeType)
      f_type = FunctionType(self.imageDomType, self.imageRangeType)
      new_ff = FlatField(f_type, self.imageDomain)
      new_ff.setSamples([vals], false)
                                                                                                                                   
      indexes                = jarray.zeros(2, 'i')
      minmax                 = Helper.minmax(vals, indexes)
      minloc = self.cs.toReference(self.imageDomain.indexToValue([indexes[0]]))
      maxloc = self.cs.toReference(self.imageDomain.indexToValue([indexes[1]]))
      self.obsMin = [minloc[0][0], minloc[1][0], minmax[0]]
      self.obsMax = [maxloc[0][0], maxloc[1][0], minmax[1]]

      return new_ff

      
  def changeParameter(self, paramName, channelIndex, line, element):
    self.paramName         = paramName
    self.spectrumRangeType = self.spectrumRangeTypeDict[paramName]
    self.imageRangeType    = self.imageRangeTypeDict[paramName]
    self.reqUnits          = self.reqUnitDict[paramName]
    
    self.imageFunctionType = FunctionType(self.imageDomType, self.imageRangeType)
    
    self.imageField        = self.getSameImageSegmentCopy(channelIndex)
    self.observationField  = self.getObservations(line, element, true)
    
    return self.imageField, self.observationField


  def validateEarthPointOnImage(self, point):
    if point == None:
      return false
    comps = point.getComponents()
    lon = comps[0].getValue()
    lat = comps[1].getValue()
    if (Double.isNaN(lon) == 1 or Double.isNaN(lat) == 1):
      return false
    xy = self.cs.fromReference([[lat], [lon]])
    if ((xy[0][0] >= 0 and xy[0][0] <= self.gxlength) and (xy[1][0] >= 0 and xy[1][0] <= self.gylength)):
      return true
    else:
      return false


  def getGeoDomain(self):
    if self.obsLonMinMax == None:      
      self.obsLonMinMax = [-60, 60]
      self.obsLatMinMax = [-90, 90]
    return self.obsLatMinMax, self.obsLonMinMax 
    
      
  def getTimeObs(self, line, element):
    return None
    

  def getFOVangleObs(self, line, element):
    return None
    
    
  def getImageValue(self, lat, lon):
    if self.unnavigated == false:
      val = self.imageField.evaluate(RealTuple(self.cs.getReference(), [lat, lon]), Data.NEAREST_NEIGHBOR, Data.NO_ERRORS)
    else:
      cc = self.cs.fromReference([[lat],[lon]])
      val = self.imageField.evaluate(RealTuple(self.imageDomType, [cc[0][0], cc[1][0]]), Data.NEAREST_NEIGHBOR, Data.NO_ERRORS)
    return val


  def getRealTypes(self):
    return [self.obsElementType, self.obsLineType, None, self.obsChannelsType, self.imageRangeType]

  def getGeoRealTypes(self):
    return [self.obsLatitudeType, self.obsLongitudeType]

  def getObservationChannelRange(self):
    return (min(self.observationChannels), max(self.observationChannels))
    
  def getObsValidRange(self):
    if self.paramName   == "radiance":
      return [0, 150]
    elif self.paramName == "brightnessTemp":
      return [180, 320]
      
  def getNumSegments(self):
    return self.numberSegments
    
  def initMissing(self):
    class Missing:
      def __init__(self):
        pass
      def removeMissing(self, values):
        return values
      
    m = Missing()
    return m
    
  def channelIndexToFileIndex(self, dataset_index, channel_index):
    return channel_index
    
  def channelIndexToDataSetIndex(self, index):
    return 0

  
  def getSpectrumRange(self, values, paramName):
    if self.native_param == paramName:
      return values
    elif paramName == "brightnessTemp":
      return self.radianceToBrightnessTempSpectrum(values)
    elif paramName == "radiance":
      return self.brightnessTempToRadiance(values,channelIndex)

      
  def getImageRange(self, values, paramName, datasetIndex, channelIndex):
    if self.native_param == paramName:
      self.imageRangeUnit = ScaledUnit(1, Parser.parse("mW/steradian/m^2/cm^-1"), "mW/ster/m^2/cm^-1")
      return values
    elif paramName == "brightnessTemp":
      self.imageRangeUnit = Parser.parse("K")
      return self.radianceToBrightnessTemp(values,channelIndex)
    elif paramName == "radiance":
      self.imageRangeUnit = ScaledUnit(1, Parser.parse("mW/steradian/m^2/cm^-1"), "mW/ster/m^2/cm^-1")      
      return self.brightnessTempToRadiance(values,channelIndex)     
    
    
  def radianceToBrightnessTemp(self, values, channelIndex):
    # Converts radiances [mW/ster/m2/cm^-1] to BT [K]
    # Input: nu  array of wavenmbers [cm^-1]
    #        B   radiances [mW/ster/m2/cm^-1]
    # Output: bt brightness temperature in [K]
    # Paolo Antonelli
    # Wed Feb 25 16:43:05 CST 1998
                                                                                                                                                          
    c1=1.191066E-5;           #- mW/m2/ster/cm^-4
    c2=1.438833;                 #- K*cm
    wavenumber = self.observationChannels[channelIndex]       
    nu         = wavenumber
    nvals      = len(values)
    new_values = jarray.zeros(nvals, 'f')   
    for i in xrange(nvals):
      B = values[i]
      bt=c2*nu/(Math.log(((c1*nu*nu*nu)/B)+1.0));
      new_values[i] = bt
      
    return new_values
    
  def radianceToBrightnessTempSpectrum(self, values):
    # Converts radiances [mW/ster/m2/cm^-1] to BT [K]
    # Input: nu  array of wavenmbers [cm^-1]
    #        B   radiances [mW/ster/m2/cm^-1]
    # Output: bt brightness temperature in [K]
    # Paolo Antonelli
    # Wed Feb 25 16:43:05 CST 1998
                                                                                                                                                          
    c1=1.191066E-5;           #- mW/m2/ster/cm^-4
    c2=1.438833;                 #- K*cm
                                                
    nvals = len(values)
    new_values = jarray.zeros(nvals, 'f')   
    for i in xrange(nvals):
      nu = self.observationChannels[i]
      B = values[i]
      bt=c2*nu/(Math.log(((c1*nu*nu*nu)/B)+1.0));
      new_values[i] = bt
      
    return new_values
    
  def brightnessTempToRadiance(self, values, wavenumber):
    # compute radiance given wavenumbers and brightness temperature.
    #
    # Inputs:
    #	freq	  wavenumbers (Nx1) in 1/cm 
    #	bt        brightnes temperature (Nx1) in Kelvin
    # Outputs:
    #	radiance  Planck radiances (Nx1) in mW/(m2 sr. 1/cm)
    #
    # see also RADTOT.M, RAD2BT.M, TTORAD.M
    # 
    # DCT 11/11-99
    #

    # fundamental constants:
    #  (Cohen, E.R. and B.N. Taylor, The 1986 CODATA recommended values
    #  of the fundamental physical constants, Journal of Research of
    #  the National Bureau of Standard, 92(2), March-April 1987.)

    h = 6.6260755E-34;  # Planck constant in Js
    c = 2.99792458E8;   # photon speed in m/s
    k = 1.380658E-23;   # Boltzmann constant in J/K

    c1 = 2*h*c*c*1e8;
    c2 = h*c/k*1e2;
    
    freq  = wavenumber
    nvals = len(values)
    for i in xrange(nvals):
      bt = values[i]
      radiance = 1e3 * c1*(freq*freq*freq)/(Math.exp((c2*freq)/bt)-1);
      new_values[i] = radiance
      
    return new_values

  def bandNameToValue(self, text):
    return self.observationChannels[list(self.fileIndexes).index((self.bandDict[text])[0])]

  def bandValueToName(self, value):
    idx = self.observationDomain.valueToIndex([[value]])[0]
    return self.band_numbers[self.fileIndexes[idx]]

  def getInitialSubset(self):
    return Subset(None,None,None,None,step=10)


class GOES_Data(MultiSpectralData):
  def __init__(self, filename, date_str, time_str, master=None):
    MultiSpectralData.__init__(self, filename, date_str, time_str, master)
    self.invert         = true
    self.broadBand      = true
    self.numObsDataSets = 1

    # res/base_res:  vis to ir resolution
    self.ratio = [1, 1, 1, 1, 1]
    #- this all assumes "full-disk" for now.
    if self.addeRq.getBand() == '1':
      self.numObsLines, self.numObsElements = self.addeRq.getSize()
      if self.addeRq.getDesc() == 'FD' or self.addeRq.getDesc() == 'CONUS':
        self.numObsLines    /= 4
        self.numObsElements /= 4
        #- blow down vis to ir
        self.ratio = [4, 1, 1, 1, 1]

    self.res_ratio = 2

  def getObservationChannels(self):
    self.fileIndexes = None
    self.numObsChannels = len(self.observationChannels)
    self.numObsChannelsPerDataSet = [self.numObsChannels]
    self.bandDict = {}
    self.fileIndexes  = QuickSort.sort(self.observationChannels)
    cnt = 0
    for i in xrange(len(self.observationChannels)):
      band_number = self.band_numbers[i]
      center_wn = self.observationChannels[i]
      self.bandDict.update({band_number : [cnt,center_wn]})
      self.bandDict.update({center_wn : band_number})
      cnt += 1
    return self.observationChannels

  def makeRangeTypes(self):
    self.spectrumRangeTypeDict = {
      "radiance":
        RealTupleType(RealType.getRealType("Radiance_1"), RealType.getRealType("Radiance_2-6")),
      "brightnessTemp":
        RealTupleType(RealType.getRealType("Reflectance"), RealType.getRealType("BrightnessTemp"))}
                                                                                                                                   
    self.imageRangeTypeDict = {
      "radiance": RealType.getRealType("Radiance"),
      "brightnessTemp": RealType.getRealType("Refl/BT")}
                                                                                                                                   
    self.rangeTypes = []
    self.rangeTypes.append(RealType.getRealType("Radiance_1"))
    self.rangeTypes.append(RealType.getRealType("Radiance_2-6"))
    self.rangeTypes.append(RealType.getRealType("Reflectance"))
    self.rangeTypes.append(RealType.getRealType("BrightnessTemp"))

    self.reqUnitDict = {"radiance" : ["RAD", "RAD", "RAD", "RAD", "RAD"], "brightnessTemp" : ["REFL", "TEMP", "TEMP", "TEMP", "TEMP"]}


  def makeSpectrumDomainType(self):
    micrometer = ScaledUnit(0.000001, CommonUnit.meter, "\xb5m")
    self.obsChannelsType = RealType.getRealType("wavelength", micrometer)
                                                                                                                                   
  def makeSpectrumRangeType(self, paramName):
    if paramName == "radiance":
      self.spectrumRangeType = RealTupleType(self.rangeTypes[0], self.rangeTypes[1])
    elif paramName == "brightnessTemp":
      self.spectrumRangeType = RealTupleType(self.rangeTypes[2], self.rangeTypes[3])
                                                                                                                                   
  def makeImageRangeType(self, paramName):
    if paramName == "radiance":
      self.imageRangeType = RealType.getRealType("radiance")
    elif paramName == "brightnessTemp":
      self.imageRangeType = RealType.getRealType("brightnessTemp")

  def getSpectrumRange(self,values, paramName):
    vals1  = jarray.zeros(len(values), 'f')
    vals2_6 = jarray.zeros(len(values), 'f')
                                                                                                                                   
    for i in xrange(len(values)):
      vals1[i]  = Float.NaN
      vals2_6[i] = Float.NaN
                                                                                                                                   
    if paramName == "radiance":
      for i in xrange(len(values)):
        if i <= 2:
          vals1[i]  = values[i]
        else:
          vals2_6[i] = values[i]
    elif paramName == "brightnessTemp":
      tmp = Helper.GOESE_radiance_to_reflectance([0], values[0:1])
      for i in xrange(len(tmp)):
        vals1[i] = tmp[i]
                                                                                                                                   
      tmp = Helper.GOESE_radiance_to_brightnessTemperature([0,1,2,3], values[1:6])
      for i in xrange(len(tmp)):
        vals2_6[i+1] = tmp[i]
                                                                                                                                   
    return [vals1,vals2_6]

  def getImageRange(self, values, paramName, dataset_index, index):
    if paramName == "radiance":
      unit = Parser.parse("mW/m^2/micrometer/steradian")
      self.imageRangeUnit = ScaledUnit(1, unit, "mW/m^2/\xb5m/ster")
      return values
    elif paramName == "brightnessTemp":
      self.imageRangeUnit = Parser.parse("K")
      """
      if dataset_index < 1:
        return self.radianceToReflectance(values, index)
      else:
        return self.radianceToBrightnessTemp(values, index)
      """
      return values
                                                                                                                                   
  def radianceToReflectance(self, values, index):
    return Helper.GOESE_radiance_to_reflectance(index, values)
                                                                                                                                   
  def radianceToBrightnessTemp(self, values, channelIndex):
    return Helper.GOESE_radiance_to_brightnessTemperature(channelIndex, values)
                                                                                                                                   
  def bandNameToValue(self, text):
    return (self.bandDict[text])[1]
                                                                                                                                   
  def bandValueToName(self, value):
    idx = self.observationDomain.valueToIndex([[value]])[0]
    return self.band_numbers[idx]
                                                                                                                                   
  def getObsValidRange(self):
    if self.paramName   == "radiance":
      return [[0,10],[0, 100]]
    elif self.paramName == "brightnessTemp":
      return [[0,1],[100, 320]]
                                                                                                                                   
  def clone(self):
    return GOES_Data(self.filename, self.date_str, self.time_str, master=self)


class GOESW_Data(GOES_Data):
  def __init__(self, filename, date_str, time_str, master=None):
    GOES_Data.__init__(self, filename, date_str, time_str, master)
    self.instrumentName      = "GOES-W"

  def getObservationChannels(self):
    self.observationChannels = [0.65, 3.9, 6.8, 10.7, 12.0]
    self.band_numbers = ['1','2','3','4','5']
    return GOES_Data.getObservationChannels(self)

  def clone(self):
    return GOESW_Data(self.filename, self.date_str, self.time_str, master=self)
                                                                                                                                   
class GOESE_Data(GOES_Data):
  def __init__(self, filename, date_str, time_str, master=None):
    GOES_Data.__init__(self, filename, date_str, time_str, master)
    self.instrumentName      = "GOES-E"

  def getObservationChannels(self):
    self.observationChannels = [0.65, 3.9, 6.5, 10.7, 13.3]
    self.band_numbers = ['1','2','3','4','6']
    return GOES_Data.getObservationChannels(self)

  def clone(self):
    return GOESE_Data(self.filename, self.date_str, self.time_str, master=self)


class GOES10_Imager(GOES_Data):
  def __init__(self, filename, date_str, time_str, master=None):
    GOES_Data.__init__(self, filename, date_str, time_str, master)
    self.instrumentName      = "GOES-E"

  def getObservationChannels(self):
    self.observationChannels = [0.65, 3.9, 6.8, 10.7, 12.0]
    self.band_numbers = ['1','2','3','4','5']
    return GOES_Data.getObservationChannels(self)
                                                                                                                             
  def clone(self):
    return GOES10_Imager(self.filename, self.date_str, self.time_str, master=self)


class GOES10_Sounder(GOES_Data):
  def __init__(self, filename, date_str, time_str, master=None):
    GOES_Data.__init__(self, filename, date_str, time_str, master)
    self.instrumentName      = "GOES-10"
    self.ratio = [1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1]
    self.res_ratio = 1

  def getObservationChannels(self):
    self.observationChannels = [3.7, 4.0, 4.1, 4.4, 4.5, 4.6, 6.5, 7.0, 7.4, 9.7, 11.0, 12.0, 12.7, 13.4, 13.6, 14.1, 14.4, 14.7]
    self.band_numbers = ['18','17','16','15','14','13','12','11','10','9','8','7','6','5','4','3','2','1']
    return GOES_Data.getObservationChannels(self)

  def makeRangeTypes(self):
    self.spectrumRangeTypeDict = {
      "radiance": RealType.getRealType("Radiance"),
      "brightnessTemp": RealType.getRealType("BrightnessTemp")}

    self.imageRangeTypeDict = {
      "radiance": RealType.getRealType("Radiance"),
      "brightnessTemp": RealType.getRealType("BrightnessTemp")}
                                                                                                                             
                                                                                                                             
    self.rangeTypes = []
    self.rangeTypes.append(RealType.getRealType("Radiance"))
    self.rangeTypes.append(RealType.getRealType("BrightnessTemp"))

    self.reqUnitDict = {"radiance" : ["RAD", "RAD", "RAD", "RAD","RAD", "RAD", "RAD", "RAD", "RAD","RAD", "RAD", "RAD", "RAD", "RAD","RAD","RAD","RAD"], "brightnessTemp" : ["TEMP", "TEMP", "TEMP", "TEMP","TEMP", "TEMP", "TEMP", "TEMP", "TEMP","TEMP", "TEMP", "TEMP", "TEMP", "TEMP","TEMP", "TEMP", "TEMP", "TEMP"]}

  def makeSpectrumRangeType(self, paramName):
    if paramName == "radiance":
      self.spectrumRangeType = self.rangeTypes[0]
    elif paramName == "brightnessTemp":
      self.spectrumRangeType = self.rangeTypes[1]

  def getObsValidRange(self):
    if self.paramName   == "radiance":
      return [0, 100]
    elif self.paramName == "brightnessTemp":
      return [100, 320]

  def getInitialSubset(self):
    return Subset(None,None,None,None,step=2)


  def clone(self):
    return GOES10_Sounder(self.filename, self.date_str, self.time_str, master=self)


    
class MsgData(MultiSpectralData):
  def __init__(self, filename, date_str, time_str, master=None):
    MultiSpectralData.__init__(self, filename, date_str, time_str, master)
    self.instrumentName = "MSG"
    self.invert = true
    self.broadBand = true
    self.numObsDataSets = 1
    self.nav_offset = 4
    
  def getObservationChannels(self):
    self.fileIndexes = None
    self.observationChannels = [0.6, 0.8, 1.6, 3.9, 6.2, 7.3, 8.7, 9.7, 10.8, 12.0, 13.4]
    self.numObsChannels = len(self.observationChannels)
    self.numObsChannelsPerDataSet = [self.numObsChannels]
    self.bandDict = {}
    self.band_numbers = ['1','2','3','4','5','6','7','8','9','10','11']
    self.fileIndexes = QuickSort.sort(self.observationChannels)
    cnt = 0
    for i in xrange(len(self.observationChannels)):
      band_number = self.band_numbers[i]
      center_wn = self.observationChannels[i]
      self.bandDict.update({band_number : [cnt,center_wn]})
      self.bandDict.update({center_wn : band_number})
      cnt += 1
    
    self.ratio = [1 for i in xrange(self.numObsChannels)]
    return self.observationChannels  
    
  def makeRangeTypes(self):
    self.spectrumRangeTypeDict = {
      "radiance": 
        RealTupleType(RealType.getRealType("Radiance_1-3"), RealType.getRealType("Radiance_4-11")),
      "brightnessTemp": 
        RealTupleType(RealType.getRealType("Reflectance"), RealType.getRealType("BrightnessTemp"))}
        
    self.imageRangeTypeDict = {
      "radiance": RealType.getRealType("Radiance"),
      "brightnessTemp": RealType.getRealType("Refl/BT")}
        
    self.rangeTypes = []
    self.rangeTypes.append(RealType.getRealType("Radiance_1-3"))
    self.rangeTypes.append(RealType.getRealType("Radiance_4-11"))
    self.rangeTypes.append(RealType.getRealType("Reflectance"))
    self.rangeTypes.append(RealType.getRealType("BrightnessTemp"))

    self.reqUnitDict = {"radiance" : ["RAD","RAD","RAD","RAD","RAD","RAD","RAD","RAD","RAD","RAD","RAD"],
                        "brightnessTemp" : ["REFL", "REFL", "REFL", "TEMP","TEMP","TEMP","TEMP","TEMP","TEMP","TEMP","TEMP"]}

    
  def makeSpectrumDomainType(self):
    micrometer = ScaledUnit(0.000001, CommonUnit.meter, "\xb5m")
    self.obsChannelsType = RealType.getRealType("wavelength", micrometer)
    
  def makeSpectrumRangeType(self, paramName):
    if paramName == "radiance":
      self.spectrumRangeType = RealTupleType(self.rangeTypes[0], self.rangeTypes[1])
    elif paramName == "brightnessTemp":
      self.spectrumRangeType = RealTupleType(self.rangeTypes[2], self.rangeTypes[3])
      
  def makeImageRangeType(self, paramName):
    if paramName == "radiance":
      self.imageRangeType = RealType.getRealType("radiance")
    elif paramName == "brightnessTemp":
      self.imageRangeType = RealType.getRealType("brightnessTemp")
      
  def getSpectrumRange(self,values, paramName):      
    vals1_3  = jarray.zeros(len(values), 'f')
    vals4_11 = jarray.zeros(len(values), 'f')
    
    for i in xrange(len(values)):
      vals1_3[i]  = Float.NaN 
      vals4_11[i] = Float.NaN
            
    if paramName == "radiance":
      for i in xrange(len(values)):
        if i <= 2:
          vals1_3[i]  = values[i]
        else:
          vals4_11[i] = values[i]
    elif paramName == "brightnessTemp":
      tmp = Helper.msg_radiance_to_reflectance([0,1,2], values[0:3])
      for i in xrange(len(tmp)):
        vals1_3[i] = tmp[i]
        
      tmp = Helper.msg_radiance_to_brightnessTemperature([0,1,2,3,4,5,6,7], values[4:12])
      for i in xrange(len(tmp)):
        vals4_11[i+3] = tmp[i]
                   
    return [vals1_3,vals4_11]
    
  def getImageRange(self, values, paramName, dataset_index, index):
    if paramName == "radiance":
      unit = Parser.parse("mW/m^2/micrometer/steradian")
      self.imageRangeUnit = ScaledUnit(1, unit, "mW/m^2/\xb5m/ster")
      return values
    elif paramName == "brightnessTemp":
      self.imageRangeUnit = Parser.parse("K")
      return values
        
  def radianceToReflectance(self, values, index):
    return Helper.msg_radiance_to_reflectance(index, values)
    
  def radianceToBrightnessTemp(self, values, channelIndex):
    return Helper.msg_radiance_to_brightnessTemperature(channelIndex, values)
    
  def bandNameToValue(self, text):
    return (self.bandDict[text])[1]
    
  def bandValueToName(self, value):
    idx = self.observationDomain.valueToIndex([[value]])[0]
    return self.band_numbers[idx]
  
  def getObsValidRange(self):
    if self.paramName   == "radiance":
      return [[0,10],[0, 100]]
    elif self.paramName == "brightnessTemp":
      return [[0,1],[100, 320]]
      
  def clone(self):
    return MsgData(self.filename, self.date_str, self.time_str, master=self)


class AVHRR3_Data(MultiSpectralData):
  def __init__(self, filename, date_str, time_str, master=None):
    MultiSpectralData.__init__(self,  filename, date_str, time_str, master)
    self.instrumentName = "AVHRR"
    self.broadBand = true
    self.numObsDataSets = 1

    if master == None:
      self.instrumentName = "AVHRR"
    else:
      self.day = master.day
                                                                                                                                         
  def getObservationUnit(self):
    self.observationUnit.append(Parser.parse("K"))
                                                                                                                                         
  def getDayNight(self):
    day = false
    return day
                                                                                                                                         
  def getObservationChannels(self):
    self.day = self.getDayNight()
    channels = []
    self.band_numbers = []
    ff       = open(hydra_path+'/ancillary/avhrr/avhrr_3', 'r')
    lines    = ff.readlines()
    nlines   = len(lines)
    self.bandDict = {}
    cnt = 0
    for i in xrange(nlines):
      if lines[i].startswith('!'):
        pass
      else:
        fields      = lines[i].split()
        band_number = fields[0]
        center_wn   = fields[1]
        low_wn      = fields[2]
        hi_wn       = fields[3]
        center_wn   = float(center_wn)
        if band_number == '3A':
          if self.day == false:
            continue
        if band_number == '3B':
          if self.day == true:
            continue
        self.band_numbers.append(band_number)
        channels.append(center_wn)
        self.bandDict.update({band_number : [cnt,center_wn]})
        self.bandDict.update({center_wn : band_number})
        cnt += 1
                                                                                                                                         
    self.fileIndexes = QuickSort.sort(channels)
                                                                                                                                         
    self.numObsChannels = len(channels)
    self.ratio = [1 for i in xrange(self.numObsChannels)]

    return channels
                                                                                                                                         
  def getSpectrumRange(self, values, paramName):
    if paramName == "brightnessTemp":

      vals1_2  = jarray.zeros(len(values), 'f')
      vals3_5  = jarray.zeros(len(values), 'f')
                                                                                                                                         
      for i in xrange(len(values)):
        if i <= 1:
          vals1_2[i] = values[i]
          vals3_5[i] = Float.NaN
        else:
          vals1_2[i] = Float.NaN
          vals3_5[i] = values[i]
                                                                                                                                         
      return [vals1_2, vals3_5]

  def getImageRange(self, values, paramName, dataset_index, index):
    if paramName == "radiance":
      unit = Parser.parse("W/m^2/micrometer/steradian")
      self.imageRangeUnit = ScaledUnit(1, unit, "W/m^2/\xb5m/ster")
      return Helper.unpack(values, self.scaler_s[dataset_index][index], self.offset_s[dataset_index][index])

    elif paramName == "brightnessTemp":
      self.imageRangeUnit = Parser.parse("K")
      return Helper.unpack(values, self.scale_s[dataset_index][index], self.offset_s[dataset_index][index])
                                                                                                                                         
  def makeRangeTypes(self):
    self.spectrumRangeTypeDict = {
      "radiance":
        RealTupleType(RealType.getRealType("Radiance_1-2"), RealType.getRealType("Radiance_3-5")),
      "brightnessTemp":
        RealTupleType(RealType.getRealType("Albedo*100"), RealType.getRealType("BrightnessTemp"))}
                                                                                                                                         
                                                                                                                                         
    self.imageRangeTypeDict = {
      "radiance": RealType.getRealType("Radiance"),
      "brightnessTemp": RealType.getRealType("Albedo/BT")}
    self.rangeTypes = []
    self.rangeTypes.append(RealType.getRealType("Radiance_1-2"))
    self.rangeTypes.append(RealType.getRealType("Radiance_3-5"))
    self.rangeTypes.append(RealType.getRealType("Albedo*100"))
    self.rangeTypes.append(RealType.getRealType("BrightnessTemp"))

    self.reqUnitDict = {"radiance" : ["BRIT", "BRIT", "RAD", "RAD", "RAD"], "brightnessTemp" : ["BRIT", "BRIT", "TEMP", "TEMP", "TEMP"]}
                                                                                                                                         
  def makeSpectrumDomainType(self):
    micrometer = ScaledUnit(0.000001, CommonUnit.meter, "\xb5m")
    self.obsChannelsType = RealType.getRealType("wavelength", micrometer)
                                                                                                                                         
  def makeSpectrumRangeType(self, paramName):
    if paramName == "radiance":
      self.spectrumRangeType = RealTupleType(self.rangeTypes[0], self.rangeTypes[1])
    elif paramName == "brightnessTemp":
      self.spectrumRangeType = RealTupleType(self.rangeTypes[2], self.rangeTypes[3])
                                                                                                                                         
  def makeImageRangeType(self, paramName):
    if paramName == "radiance":
      self.imageRangeType = RealType.getRealType("radiance")
    elif paramName == "brightnessTemp":
      self.imageRangeType = RealType.getRealType("brightnessTemp")
  def bandNameToValue(self, text):
    return self.observationChannels[(self.bandDict[text])[0]]
                                                                                                                                         
  def bandValueToName(self, value):
    idx = self.observationDomain.valueToIndex([[value]])[0]
    return self.band_numbers[idx]
                                                                                                                                         
  def getObsValidRange(self):
    if self.paramName   == "radiance":
      return [[0,500],[0, 10]]
    elif self.paramName == "brightnessTemp":
      return [[0,100],[-90, 50]]
                                                                                                                                         
  def clone(self):
    return AVHRR3_Data(self.filename, self.date_str, self.time_str, master=self)



#-- ADDE url helper
#----------------------------------------------------
class ADDErequest:
  def __init__(self, url):
    self.url    = url+"&compress=gzip&port=112"+"&auto-upcase=true"
    param_string = (self.url.split('?'))[1]
    self.sub_strings = param_string.split('&')
                                                                                                                                   
    self.size_i = -1
    self.mag_i  = -1
    self.band_i = -1
    self.latlon_i = -1
    self.linele_i = -1
    self.unit_i   = -1
    self.desc_i   = -1
    self.group_i = -1
                                                                                                                                   
    for i in xrange(len(self.sub_strings)):
      str = self.sub_strings[i]
      if str.startswith('size'):   self.size_i   = i
      if str.startswith('mag'):    self.mag_i    = i
      if str.startswith('band'):   self.band_i   = i
      if str.startswith('latlon'): self.latlon_i = i
      if str.startswith('linele'): self.linele_i = i
      if str.startswith('unit'):   self.unit_i   = i
      if str.startswith('descr'):   self.desc_i   = i
      if str.startswith('group'):  self.group_i = i
                                                                                                                                   
    #- size:
    str = self.sub_strings[self.size_i]
    subs = str.split('=')
    le = subs[1].split(' ')
    self.lines = int(le[0])
    self.elems = int(le[1])
    #- mag:
    str = self.sub_strings[self.mag_i]
    subs = str.split('=')
    mm = subs[1].split(' ')
    self.lmag = int(mm[0])
    self.emag = int(mm[1])
    #- group:
    str = self.sub_strings[self.group_i]
    subs = str.split('=')
    self.group = subs[1]
    #- desc:
    str = self.sub_strings[self.desc_i]
    subs = str.split('=')
    self.desc = subs[1]
    #-band:
    str = self.sub_strings[self.band_i]
    subs = str.split('=')
    self.band = subs[1]
    #-unit:
    str = self.sub_strings[self.unit_i]
    subs = str.split('=')
    self.unit = subs[1]
                                                                                                                                   
  def getSize(self):
    return [self.lines, self.elems]
                                                                                                                                   
  def getMag(self):
    return [self.lmag, self.emag]
                                                                                                                                   
  def getGroup(self):
    return self.group

  def getDesc(self):
    return self.desc

  def getBand(self):
    return self.band
                                                                                                                                   
  def newRequest(self, lines=None, elems=None, lmag=None, emag=None, band=None, lin=None, ele=None, unit=None):
    if lines != None: self.lines = lines
    if elems != None: self.elems = elems
    if lmag  != None: self.lmag  = lmag
    if emag  != None: self.emag  = emag
    if band  != None: self.band  = band
    if unit  != None: self.unit  = unit

    new_url = self.url.replace(self.sub_strings[self.size_i], 'size='+str(self.lines)+' '+str(self.elems))
    new_url = new_url.replace(self.sub_strings[self.mag_i], 'mag='+str(self.lmag)+' '+str(self.emag))
    new_url = new_url.replace(self.sub_strings[self.band_i], 'band='+self.band)
    new_url = new_url.replace(self.sub_strings[self.unit_i], 'unit='+self.unit)
                                                                                                                                    
    if (lin != None and ele != None):
      if self.latlon_i != -1:
        new_url = new_url.replace(self.sub_strings[self.latlon_i], 'linele='+str(lin)+' '+str(ele)+' '+'A'+'&place=upperleft')
      else:
        new_url = new_url.replace(self.sub_strings[self.linele_i], 'linele='+str(lin)+' '+str(ele)+' '+'A'+'&place=upperleft')

    return new_url
